DATA_SECTION
  // Read data file
  init_int nc
  init_matrix Cdata(1,nc,1,2)  // Year | C
  init_int ni
  init_matrix Idata(1,ni,1,2)  // Year | I
  // Vectors
  ivector Cyear(1,nc)
  ivector Iyear(1,ni)
  vector C(1,nc)
  vector I(1,ni)
  // Constants
  number halfnlog2pi
  vector logI(1,ni)

PARAMETER_SECTION
  // Estimated
  init_number logr
  init_number logk
  init_number loga
  init_number logq
  init_number logsigma
  // Derived
  number r
  number k
  number a
  sdreport_number q
  number sigma
  // Updated
  vector B(1,nc)
  vector Ifit(1,ni)
  number RSS
  // Objfun
  objective_function_value neglogL

PRELIMINARY_CALCS_SECTION
  // Data
  Cyear = (ivector) column(Cdata,1);
  C = column(Cdata,2);
  Iyear = (ivector) column(Idata,1);
  I = column(Idata,2);
  halfnlog2pi = 0.5*ni*log(2.0*M_PI);
  logI = log(I);

PROCEDURE_SECTION
  get_fit();
  get_neglogL();

FUNCTION get_fit
  r = mfexp(logr);
  k = mfexp(logk);
  a = mfexp(loga);
  q = mfexp(logq);
  sigma = mfexp(logsigma);
  B(1) = k * a;
  for(int t=1; t<=nc-1; t++)
    B(t+1) = sfabs(B(t) + r*B(t)*(1-B(t)/k) - C(t));
  Ifit = q*B;

FUNCTION get_neglogL
  RSS = sum(square(logI-log(Ifit)));
  neglogL = halfnlog2pi + ni*log(sigma) + RSS/(2.0*square(sigma));
