//            *****************************************************\n";
//            *                                                   *\n";
//            *      Generalised Bayesian Age-Structured          *\n";
//            *    Statistical Catch-at-Age and Length Model      *\n";
//            *                                                   *\n";
//            *  Designed by R. Hilborn, A. Parma, M. Maunder     *\n";
//            *                 and b. eRnSt                      *\n";
//            *          (last update 2/19/03) B.                 *\n";
//            * Programmed by M. Maunder, B. Ernst and A. Parma   *\n";                           
//            *   Using AD Model Builder (C) Otter Research Ltd.  *\n";
//            *                                                   *\n";
//            *****************************************************\n";
//
//  OjO: For the structured data there must be consistency between likihood flag and index number in the data

DATA_SECTION
 // debug = 0 --> no debugging
 // debug = 1 --> debugging
 // debug = 2 --> use parameter file
  init_int debug
 LOCAL_CALCS
  if(debug==2)
    {  
      ad_comm::change_pinfile_name("colera31.par"); 
    }
 END_CALCS

 
 !!if (debug==1){char mark; cout<<"starting debug /n"; cin>>mark; }

  // ** options **
  init_int useWA //use annual weight at age data
  init_number useBound //Value between (0 , 1), if <= 0 or >= 1 -> is set to 0.99
  init_int ini_Gear //0=read in 1=gear 1 StartYear 2=gear 2 ....
  init_int proj_gears //0=read in, 1=last years proportions

  // ** dimentioning parameters **
  init_int NCPUEindex   //this determines how many different q's we will have
  init_int Nsurveyindex 
  init_int StartYear
  init_int EndYear
  init_int Nages
  init_int PoolAge  // C@A data
  init_int FirstAge // C@A data
  init_int FirstL
  init_int IncL
  init_int Nlens
  init_int PoolLength //length at which catch_at_length get's pooled
  init_int Nmethods  // how many different fishing gears do we have
  init_int Nsexes
 !!if (debug==1){char mark; cout<<"Nsexes= "<<Nsexes<<endl; cin>>mark; }

  // ** Projection Parameters **
  init_int StrategyType //1:constant catch 2:harvest rate
  init_int ProjectYear
  init_number StartStrategy
  init_number EndStrategy
  init_number StepStrategy
  init_vector strategyuproportion(1,Nmethods)
  init_number assessment_cv
  init_number assessment_rho
 !!if (debug==1){char mark; cout<<"assessment_rho= "<<assessment_rho<<endl; cin>>mark; }

  // ** Priors **       
  init_vector R0_prior(1,7)  //1.Phase| 2.LB| 3.UB| 4.Prior_Type| 5.Mean| 6.Cv| 7.Initial_Guess
  init_vector h_prior(1,7)   //                       Priortype 0:uniform (or non defined), 1:normal, 2:lognormal
  init_matrix M_prior(1,Nsexes,1,7)
  init_vector p_log_InitialDev(1,7)
  init_vector p_log_RecDev(1,7)
  init_vector Rinit_prior(1,7)
  init_matrix uinit_prior(1,Nsexes,1,7)
  init_matrix p_plusscale(1,Nsexes,1,7)
  init_matrix p_Sfullest(1,Nmethods,1,7) 
  init_matrix p_Sfulldelta(1,Nmethods,1,7) 
  init_matrix log_varLest_prior(1,Nmethods,1,7) 
  init_matrix log_varRest_prior(1,Nmethods,1,7) 
  init_matrix errSfull_prior(1,Nmethods,1,7)
  init_matrix errvarL_prior(1,Nmethods,1,7)
  init_matrix errvarR_prior(1,Nmethods,1,7)
  init_matrix log_qCPUE_prior(1,NCPUEindex,1,7)
  init_matrix qCPUEerr_prior(1,NCPUEindex,1,7)
  init_matrix log_qsurvey_prior(1,Nsurveyindex,1,7)
  init_matrix surveySfull_prior(1,Nsurveyindex,1,7)
  init_matrix p_surveySfulldelta(1,Nsurveyindex,1,7)
  init_matrix log_surveyvarL_prior(1,Nsurveyindex,1,7)
  init_matrix log_surveyvarR_prior(1,Nsurveyindex,1,7)

  init_vector tempfind_prior(1,7);
 !!if (debug==1){char mark; cout<<"tempfind_prior= "<<tempfind_prior<<endl; cin>>mark; }

  // ** Likelihood **
  init_ivector CPUE_LikeType(1,NCPUEindex)
  init_ivector CA_LikeType(1,Nmethods)
  init_ivector CL_LikeType(1,Nmethods)
  init_ivector survey_LikeType(1,Nsurveyindex)  // accepts 0/2/-2
  init_ivector survey_IndexType(1,Nsurveyindex) //1=weight 2=numbers
  init_ivector survey_vType(1,Nsurveyindex)     //1=age 2=length
  init_ivector surveynosexCL_LikeType(1,Nsurveyindex)
  init_ivector surveyCL_LikeType(1,Nsurveyindex)
  init_ivector surveyCA_LikeType(1,Nsurveyindex)
 !!if (debug==1){char mark; cout<<"surveyCA_LikeType= "<<surveyCA_LikeType<<endl; cin>>mark; }

  // ** fixed parameters **
  init_vector bi(1,Nsexes)
  init_vector bii(1,Nsexes)
  init_vector linf(1,Nsexes)
  init_vector k(1,Nsexes)
  init_vector t0(1,Nsexes)
  init_ivector type_sd_rel(1,Nsexes)   // this was changed to ivector to acomodate a flag to control type of length-age relationship --WAS BEFORE :init_vector Brody(1,Nsexes)
  init_vector Lmean1(1,Nsexes)
  init_vector LmeanMax(1,Nsexes)
  init_vector Lsd1(1,Nsexes)
  init_vector Lsdn(1,Nsexes)
  init_vector mat(1,Nages)
  init_vector RecFraction(1,Nsexes)
  init_matrix vinit_in(1,Nsexes,1,Nages) 
  init_matrix Aerror(1,Nages,1,Nages)
  init_int NWA
  init_matrix WA(1,NWA,1,Nages+2) //Year Sex a1 a2 a3 ..
 !!if (debug==1){char mark; cout<<"WA= "<<WA<<endl; cin>>mark; }

  // ** data **
  init_matrix C(1,Nmethods,StartYear,EndYear)
  init_int NCPUE 
  init_matrix CPUE(1,NCPUE,1,5)
  init_int Nsurvey 
  init_matrix survey(1,Nsurvey,1,4)
!!if (debug==1){char mark; cout<<"survey= "<<survey<<endl; cin>>mark; }
  init_int NCA
  init_matrix CA(1,NCA,1,Nages*Nsexes+3) 
  init_int NsurveyCA
  init_matrix surveyCA(1,NsurveyCA,1,Nages*Nsexes+3) 
!!if (debug==1){char mark; cout<<"surveyCA= "<<surveyCA<<endl; cin>>mark; }
  init_int NCL
  init_matrix CL(1,NCL,1,Nlens*Nsexes+3) 
  init_int NsurveyCL
  init_matrix surveyCL(1,NsurveyCL,1,Nlens*Nsexes+3) 
  init_int NsurveynosexCL
  init_matrix surveynosexCL(1,NsurveynosexCL,1,Nlens+3) 
 !!if (debug==1){char mark; cout<<"last line: surveynosexCL= "<<surveynosexCL<<endl; cin>>mark; }
  int Nqsurveys   //number of survey indices actually used, counted below

  !! Nqsurveys = Nsurveyindex; 
  ivector used_surveys(1,Nqsurveys)  //indices of used surveys set below
   
 LOCAL_CALCS
  for (i=1;i<=Nsurveyindex;i++)
   {
     used_surveys(i) = i;
   }
 END_CALCS

// ********************************************************
// STUFF for Survey Predictions and Likelihood calculations
// ********************************************************
 ivector nObsSIndex(1,Nsurveyindex) // for each Survey index (all) how many Obs do we have
 vector zeroSIndextmp(1,Nsurveyindex)
 ivector whatSIndex(1,Nsurveyindex);
 int nUsedSIndex;

 LOCAL_CALCS
  zeroSIndextmp=0.0; 
  nUsedSIndex=0;
  whatSIndex=0;
  int indexx;
  for (i=1;i<=Nsurveyindex;i++)
     {
       if(survey_LikeType(i) > 0) 
         {
	   zeroSIndextmp(i)=1.0;	        // generate vector with 1 and 0 for the likelihood
           nUsedSIndex+=1;			// how many indeces are used?
           whatSIndex(nUsedSIndex)=i;    	// Which ones?

         }
     }
     

  indexx=0;   
  // we assume that each the Nsurveyindex is represented in the data

  nObsSIndex=0;
  for (i=1;i<=Nsurvey;i++)
     {
       indexx=int(survey(i,1)); 
       nObsSIndex(indexx)++;    //computes how many observation are per         
     }  
    
 END_CALCS
  
  matrix yearSIndex(1,Nsurveyindex,1,nObsSIndex)   // This contains info for which year we have data for index i.
  matrix ObsSIndex(1,Nsurveyindex,1,nObsSIndex)    // This contains Survey estimate, index i.
  matrix ObsCvSIndex(1,Nsurveyindex,1,nObsSIndex)  // This contains CV of Survey estimate, index i.

 LOCAL_CALCS
  ivector indexpos(1,Nsurveyindex);
  int oldindexx;
  indexpos=0;
  indexx=0;
  for (i=1;i<=Nsurvey;i++)
     {
          indexx=int(survey(i,1));  
          indexpos(indexx)+=1;   
          yearSIndex(indexx,indexpos(indexx)) = int(survey(i,2)); 
          ObsSIndex(indexx,indexpos(indexx)) = survey(i,3);  // get the survey index observed data
          ObsCvSIndex(indexx,indexpos(indexx)) = survey(i,4); // get survey index observed CV data
     }
 END_CALCS
 
// CPUE stuff
// ================

 int nUsedCPUE;                    	 // how many Surveys Indeces are turned on (Likelihoods), out of Nsurveyindex
 ivector nObsCPUE(1,NCPUEindex)    	 // for each Survey index (all) how many Obs do we have
 vector zeroCPUEtmp(1,NCPUEindex) 	 // this way all the <= 0 flaged likelihoods are set to zero during the estimation
 ivector whatCPUE(1,NCPUEindex);
 ivector whatSelecforCPUE(1,NCPUEindex); // keep track of selectivity or gear index 
 int prflagCPUEtmp 

 LOCAL_CALCS
  prflagCPUEtmp=0;  
  zeroCPUEtmp=0.0;
  nUsedCPUE=0;
  
  
  for (i=1;i<=NCPUEindex;i++)
     {
       if(CPUE_LikeType(i) > 0) 
          { 
            prflagCPUEtmp=1;  
            zeroCPUEtmp(i)=1.0;
            nUsedCPUE+=1;		// how many indeces are used?
            whatCPUE(nUsedCPUE)=i;    	// keeps track of CPUE index from the data
          }
     }

  indexx=0;
  nObsCPUE=0;
  for (i=1;i<=NCPUE;i++)
     {
       indexx=int(CPUE(i,1)); 
       nObsCPUE(indexx)++;            
     }

 END_CALCS
  matrix yearCPUE(1,NCPUEindex,1,nObsCPUE)  //ragged stuff 
  matrix ObsCPUE(1,NCPUEindex,1,nObsCPUE)
  matrix ObsCvCPUE(1,NCPUEindex,1,nObsCPUE)

 LOCAL_CALCS
  ivector indexpos2(1,NCPUEindex);
  indexx=1;
  indexpos2=0;
  i=0	;
  for (i=1;i<=NCPUE;i++)
     {
          indexx=int(CPUE(i,1));  
          indexpos2(indexx)+=1;   
          whatSelecforCPUE(indexx)=CPUE(i,2);
  	  yearCPUE(indexx,indexpos2(indexx)) = int(CPUE(i,3)); 
          ObsCPUE(indexx,indexpos2(indexx)) = CPUE(i,4);  
          ObsCvCPUE(indexx,indexpos2(indexx)) = CPUE(i,5); 
     }

 END_CALCS

// ************************************************************
// STUFF for Survey CA Predictions and Likelihood calculations
// ************************************************************
 // howManySCA    <-- how many different indices are there in the CA data.
 // whichSCA      <-- which indices are there in the data
 // whichLikeSCA  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCA
 // nUsedSCA      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroSCAtmp    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearSCA       <-- DATA: years for which catch at age is available per index
 // ObsSsizeSCA   <-- DATA: sample size
 // ObsSCA        <-- DATA: proportions at age

   int howManySCA; 
   int nUsedSCA;   

 LOCAL_CALCS
   imatrix howmanytmp(1,3,1,10); // DUMMY: this should take care of the problem when the user does not enter the CA data in an orderly fashion
   int j;
   int itmp;
   howmanytmp.initialize();     
   howManySCA=0;
   nUsedSCA=0;
   itmp=0;
   j=0;
   for (i=1;i<=NsurveyCA;i++)
      {
        itmp=int(surveyCA(i,1));
        howmanytmp(1,itmp)=1;            // This helps to compute how many different indices are in the Survey CA data
        howmanytmp(2,itmp)+=1;           // This counts how many observations per index
      }    
    for (i=1;i<=10;i++) 
       {
         if(howmanytmp(1,i)==1)          
	   {
	     j++;
	     howmanytmp(3,i)=j;          
             howManySCA+=1;
	   } 
       }	 

 END_CALCS

  ivector nObsSCA(1,howManySCA);     // for each Survey index (all) how many Obs do we have
  ivector whichSCA(1,howManySCA);    // which index are active 
  vector zeroSCAtmp(1,howManySCA);   // switches for the likelihoods 
  int prflagSCAtmp

   
 LOCAL_CALCS
  int jj;
  nObsSCA.initialize();
  zeroSCAtmp=0.0;
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) // this should go over all likelihood flags
     {
       if(surveyCA_LikeType(i) > 0) 
          { 
            nUsedSCA+=1;			
            prflagSCAtmp=1;  
            zeroSCAtmp(i)=1.0;
          }
     }
 
 END_CALCS   

  ivector whichLikeSCA(1,howManySCA);
 LOCAL_CALCS
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) // this should go over all likelihood flags
     {
      if(howmanytmp(1,i)==1)
        {      
          j+=1;	
          whichSCA(j)=i;    	
	  if(surveyCA_LikeType(i) > 0) 
	     {
	       jj+=1;
	       whichLikeSCA(jj)=j;
	     }  
          nObsSCA(j)=howmanytmp(2,i);
        }    
     }
 END_CALCS

  matrix yearSCA(1,howManySCA,1,nObsSCA) // years for each observation 
  3darray ObsSCA(1,howManySCA,1,nObsSCA,1,PoolAge*Nsexes)
  matrix ObsSsizeSCA(1,howManySCA,1,nObsSCA)

 LOCAL_CALCS

  ivector indexpos3(1,howManySCA);
  itmp=0;
  indexpos3.initialize();
  for (i=1;i<=NsurveyCA;i++)  // this goes over each row of the Survey Catch@Age data
     {
        indexx = int(surveyCA(i,1));  
        indexx = howmanytmp(3,indexx);
        indexpos3(indexx)+=1;
        itmp=indexpos3(indexx);
        yearSCA(indexx,itmp)=surveyCA(i,2);
        ObsSsizeSCA(indexx,itmp)=surveyCA(i,3); 
  
         for (int sex=1;sex<=Nsexes; sex++)
            {
    	      for (int age=1; age<=PoolAge;age++)
		   {
		     
		     ObsSCA(indexx,itmp,PoolAge*(sex-1)+age)=surveyCA(i,Nages*(sex-1)+age+3);
		   }
	       for (age=PoolAge+1; age<=Nages;age++)
		   {
		     ObsSCA(indexx,itmp,PoolAge*sex)+=surveyCA(i,Nages*(sex-1)+age+3);
		   }
	     }
       }

 END_CALCS

// *******************************************
// STUFF for FISHERY CA Likelihood calculation
// *******************************************
 
 // howManyCA    <-- how many different indices are there in the Fleet CA data.
 // whichCA      <-- which indices are there in the data
 // whichLikeCA  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCA
 // nUsedCA      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroCAtmp    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearCA       <-- DATA: years for which catch at age is available per index
 // ObsSsizeCA   <-- DATA: sample size
 // ObsCA        <-- DATA: proportions at age

//======================================================================================
   int howManyCA; 
   int nUsedCA;   

 LOCAL_CALCS
   imatrix howmanytmpCA(1,3,1,10); // DUMMY: this should take care of the problem when the user does not enter the CA data in an orderly fashion
   howmanytmpCA.initialize();     
   howManyCA=0;
   nUsedCA=0;
   itmp=0;
   j=0;
   for (i=1;i<=NCA;i++)
      {
        itmp=int(CA(i,1));
        howmanytmpCA(1,itmp)=1;            // This helps to compute how many different indices are in the Survey CA data
        howmanytmpCA(2,itmp)+=1;           // This counts how many observations per index
      }    
    for (i=1;i<=10;i++) 
       {
         if(howmanytmpCA(1,i)==1)   
	   {
	     j++;
	     howmanytmpCA(3,i)=j;          // This helps to collapse fix dimension to howManyCA
             howManyCA+=1;
	   } 
       }	 
 END_CALCS

  ivector nObsCA(1,howManyCA);     // for each Survey index (all) how many Obs do we have
  ivector whichCA(1,howManyCA);    // which index are active 
  vector zeroCAtmp(1,howManyCA);   // switches for the likelihoods 
  int prflagCAtmp

   
 LOCAL_CALCS
  nObsCA.initialize();
  zeroCAtmp=0.0;
  j=0;
  jj=0;
  for (i=1;i<=Nmethods;i++) // this should go over all likelihood flags
     {
       if(CA_LikeType(i) > 0) 
          { 
            nUsedCA+=1;			
            prflagCAtmp=1;  
            zeroCAtmp(i)=1.0;
          }
     }
 END_CALCS   

  ivector whichLikeCA(1,howManyCA);
 LOCAL_CALCS
  j=0;
  jj=0;
  whichLikeCA.initialize();
  for (i=1;i<=Nmethods;i++) // this should go over all likelihood flags  OJO: this should goover Nmethods
     {
      if(howmanytmpCA(1,i)==1)
        {      
          j+=1;	
          whichCA(j)=i;    	
	  if(CA_LikeType(i) > 0) 
	     {
	       jj+=1;
	       whichLikeCA(jj)=j;
	     }  
          nObsCA(j)=howmanytmpCA(2,i);
        }    
     }

 END_CALCS

  matrix yearCA(1,howManyCA,1,nObsCA) // years for each observation 
  3darray ObsCA(1,howManyCA,1,nObsCA,1,PoolAge*Nsexes)
  matrix ObsSsizeCA(1,howManyCA,1,nObsCA)

 LOCAL_CALCS


  ivector indexpos3CA(1,howManyCA);
  itmp=0;
  indexx=0;
  indexpos3CA.initialize();
  for (i=1;i<=NCA;i++)  // this goes over each row of the Survey Catch@Age data
     {
        indexx = int(CA(i,1));  
        indexx = howmanytmpCA(3,indexx);
        indexpos3CA(indexx)+=1;
        itmp=indexpos3CA(indexx);
        yearCA(indexx,itmp)=CA(i,2);
        ObsSsizeCA(indexx,itmp)=CA(i,3); 
  
         for (int sex=1;sex<=Nsexes; sex++)
            {
    	      for (int age=1; age<=PoolAge;age++)
		   {
		     ObsCA(indexx,itmp,PoolAge*(sex-1)+age)=CA(i,Nages*(sex-1)+age+3);
		   }
	       for (age=PoolAge+1; age<=Nages;age++)
		   {
		     ObsCA(indexx,itmp,PoolAge*sex)+=CA(i,Nages*(sex-1)+age+3);
		   }
	     }
       }

 END_CALCS

//=======================================================================================


 int iPoolLength
 LOCAL_CALCS
  iPoolLength=((PoolLength-FirstL)/IncL)+1;  
 END_CALCS
// ************************************************************
// STUFF for Survey CL Predictions and Likelihood calculations
// ************************************************************
 // howManySCL    <-- how many different indices are there in the CL data.
 // whichSCL      <-- which indices are there in the data
 // whichLikeSCA  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCA
 // nUsedSCL      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroSCLtmp    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearSCL       <-- DATA: years for which catch at age is available per index
 // ObsSsizeSCL   <-- DATA: sample size
 // ObsSCL        <-- DATA: proportions at age

   int howManySCL; 
   int nUsedSCL;   

 LOCAL_CALCS

   imatrix howmanytmpSCL(1,3,1,10); // DUMMY: this should take care of the problem when the user does not enter the CA data in an orderly fashion
  // int j;
  // int itmp;
   howmanytmpSCL.initialize();     
   howManySCL=0;
   nUsedSCL=0;
   itmp=0;
   j=0;
   for (i=1;i<=NsurveyCL;i++)
      {
        itmp=int(surveyCL(i,1));
        howmanytmpSCL(1,itmp)=1;    
        howmanytmpSCL(2,itmp)+=1;   
      }    
    for (i=1;i<=10;i++) 
       {
         if(howmanytmpSCL(1,i)==1)  
	   {
	     j++;
	     howmanytmpSCL(3,i)=j;  
             howManySCL+=1;
	   } 
       }	 
 END_CALCS

  ivector nObsSCL(1,howManySCL);     // for each Survey index (all) how many Obs do we have
  ivector whichSCL(1,howManySCL);    // which index are active 
  vector zeroSCLtmp(1,howManySCL);   // switches for the likelihoods 
  int prflagSCLtmp

   
 LOCAL_CALCS
//  int jj;
  nObsSCL.initialize();
  whichSCL.initialize();
  zeroSCLtmp.initialize();
  zeroSCLtmp=0.0;
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) // this should go over all likelihood flags
     {
       if(surveyCL_LikeType(i) > 0) 
          { 
            nUsedSCL+=1;			
            prflagSCLtmp=1;  
            zeroSCLtmp(i)=1.0;
          }
     }
 END_CALCS   

  ivector whichLikeSCL(1,howManySCL);
 LOCAL_CALCS
  whichLikeSCL.initialize();
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) // this should go over all likelihood flags
     {
      if(howmanytmpSCL(1,i)==1)
        {      
          j+=1;	
          whichSCL(j)=i;    	
	  if(surveyCL_LikeType(i) > 0) 
	     {
	       jj+=1;
	       whichLikeSCL(jj)=j;
	     }  
          nObsSCL(j)=howmanytmpSCL(2,i);
        }    
     }
 END_CALCS

  matrix yearSCL(1,howManySCL,1,nObsSCL) // years for each observation 
  3darray ObsSCL(1,howManySCL,1,nObsSCL,1,iPoolLength*Nsexes)
  matrix ObsSsizeSCL(1,howManySCL,1,nObsSCL)

 LOCAL_CALCS
  ivector indexposSCL(1,howManySCL);
  itmp=0;
  indexposSCL.initialize();
  for (i=1;i<=NsurveyCL;i++)  
     {
        indexx = int(surveyCL(i,1));  
        indexx = howmanytmpSCL(3,indexx);
        indexposSCL(indexx)+=1;
        itmp=indexposSCL(indexx);
        yearSCL(indexx,itmp)=surveyCL(i,2);
        ObsSsizeSCL(indexx,itmp)=surveyCL(i,3); 
  
         for (int sex=1;sex<=Nsexes; sex++)
            {
              for (int length=1; length<iPoolLength;length++)
                {
                  ObsSCL(indexx,itmp,iPoolLength*(sex-1)+length)=surveyCL(i,Nlens*(sex-1)+length+3);
      
                 }
              for (length=iPoolLength; length<=Nlens;length++)
                 {
                    ObsSCL(indexx,itmp,iPoolLength*sex)+=surveyCL(i,Nlens*(sex-1)+length+3);
                 }
            }
       }
 END_CALCS




// ********************************************************
// STUFF for FISHERY CL and Likelihood calculations
// ********************************************************
 // howManyCL    <-- how many different indices are there in the Fleet CL data.
 // whichCL      <-- which indices are there in the data
 // whichLikeCL  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCL
 // nUsedCL      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroCLtmp    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearCL       <-- DATA: years for which catch at age is available per index
 // ObsSsizeCL   <-- DATA: sample size
 // ObsCL        <-- DATA: proportions at age

//======================================================================================
   int howManyCL; 
   int nUsedCL;   

 LOCAL_CALCS
   imatrix howmanytmpCL(1,3,1,10); // DUMMY: this should take care of the problem when the user does not enter the CA data in an orderly fashion
   howmanytmpCL.initialize();     
   howManyCL=0;
   nUsedCL=0;
   itmp=0;
   j=0;
   for (i=1;i<=NCL;i++)
      {
        itmp=int(CL(i,1));
        howmanytmpCL(1,itmp)=1;            // This helps to compute how many different indices are in the Survey CA data
        howmanytmpCL(2,itmp)+=1;           // This counts how many observations per index
      }    
    for (i=1;i<=10;i++) 
       {
         if(howmanytmpCL(1,i)==1)          
	   {
	     j++;
	     howmanytmpCL(3,i)=j;          // This helps to collapse fix dimension to howManyCA
             howManyCL+=1;
	   } 
       }	 
 END_CALCS

  ivector nObsCL(1,howManyCL);     // for each Survey index (all) how many Obs do we have
  ivector whichCL(1,howManyCL);    // which index are active 
  vector zeroCLtmp(1,howManyCL);   // switches for the likelihoods 
  int prflagCLtmp

   
 LOCAL_CALCS
  nObsCL.initialize();
  zeroCLtmp=0.0;
  j=0;
  jj=0;
  for (i=1;i<=Nmethods;i++) // this should go over all likelihood flags
     {
       if(CL_LikeType(i) > 0) 
          { 
            nUsedCL+=1;			
            prflagCLtmp=1;  
            zeroCLtmp(nUsedCL)=1.0;
          }
     }
 END_CALCS   

  ivector whichLikeCL(1,howManyCL);
 LOCAL_CALCS
  j=0;
  jj=0;
  whichLikeCL.initialize();
  for (i=1;i<=Nmethods;i++) // this should go over all likelihood flags  OJO: this should goover Nmethods
     {
      if(howmanytmpCL(1,i)==1)
        {      
          j+=1;	
          whichCL(j)=i;    	
	  if(CL_LikeType(i) > 0) 
	     {
	       jj+=1;
	       whichLikeCL(jj)=j;
	     }  
          nObsCL(j)=howmanytmpCL(2,i);
        }    
     }

 END_CALCS

  matrix yearCL(1,howManyCL,1,nObsCL) // years for each observation 
  3darray ObsCL(1,howManyCL,1,nObsCL,1,iPoolLength*Nsexes)
  matrix ObsSsizeCL(1,howManyCL,1,nObsCL)

 LOCAL_CALCS
  ivector indexpos3CL(1,howManyCL);
  itmp=0;
  indexx=0;
  indexpos3CL.initialize();
  for (i=1;i<=NCL;i++)  // this goes over each row of the Survey Catch@Age data
     {
        indexx = int(CL(i,1));  
        indexx = howmanytmpCL(3,indexx);
        indexpos3CL(indexx)+=1;
        itmp=indexpos3CL(indexx);
        yearCL(indexx,itmp)=CL(i,2);
        ObsSsizeCL(indexx,itmp)=CL(i,3); 
  
           for (int sex=1;sex<=Nsexes; sex++)
            {
              for (int length=1; length<iPoolLength;length++)
                {
                  ObsCL(indexx,itmp,iPoolLength*(sex-1)+length)=CL(i,Nlens*(sex-1)+length+3);
      
                 }
              for (length=iPoolLength; length<=Nlens;length++)
                 {
                    ObsCL(indexx,itmp,iPoolLength*sex)+=CL(i,Nlens*(sex-1)+length+3);
                 }
            }
       }
 END_CALCS

//=======================================================================================

// *******************************************************************
// STUFF for no sex Survey CL Predictions and Likelihood calculations
// *******************************************************************
 // howManySCLns    <-- how many different indices are there in the nosex  CL data.
 // whichSCLns      <-- which indices are there in the data
 // whichLikeSCLns  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCA
 // nUsedSCLns      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroSCLtmpns    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearSCLns       <-- DATA: years for which catch at age is available per index
 // ObsSsizeSCLns   <-- DATA: sample size
 // ObsSCLns        <-- DATA: proportions at age
   int howManySCLns; 
   int nUsedSCLns;   

 LOCAL_CALCS

   imatrix howmanytmpSCLns(1,3,1,10); // DUMMY: this should take care of the problem when the user does not enter the CA data in an orderly fashion
  // int j;
  // int itmp;
   howmanytmpSCLns.initialize();     
   howManySCLns=0;
   nUsedSCLns=0;
   itmp=0;
   j=0;
   for (i=1;i<=NsurveynosexCL;i++)
      {
        itmp=int(surveynosexCL(i,1));
        howmanytmpSCLns(1,itmp)=1;            // This helps to compute how many different indices are in the Survey CA data
        howmanytmpSCLns(2,itmp)+=1;           // This counts how many observations per index
      }    
    for (i=1;i<=10;i++) 
       {
         if(howmanytmpSCLns(1,i)==1)          
	   {
	     j++;
	     howmanytmpSCLns(3,i)=j;          // This helps to collapse fix dimension to howManySCA
             howManySCLns+=1;
	   } 
       }	 
 END_CALCS

  ivector nObsSCLns(1,howManySCLns);     // for each Survey index (all) how many Obs do we have
  ivector whichSCLns(1,howManySCLns);    // which index are active 
  vector zeroSCLtmpns(1,howManySCLns);   // switches for the likelihoods 
  int prflagSCLtmpns

   
 LOCAL_CALCS
//  int jj;
  
  nObsSCLns.initialize();
  whichSCLns.initialize();
  zeroSCLtmpns.initialize();
  zeroSCLtmpns=0.0;
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) 
     {
       if(surveynosexCL_LikeType(i) > 0) 
          { 
            nUsedSCLns+=1;			
            prflagSCLtmpns=1;  
            zeroSCLtmpns(i)=1.0;
          }
     }
 END_CALCS   

  ivector whichLikeSCLns(1,howManySCLns);
 LOCAL_CALCS
  whichLikeSCLns.initialize();
  j=0;
  jj=0;
  for (i=1;i<=Nsurveyindex;i++) 
     {
      if(howmanytmpSCLns(1,i)==1)
        {      
          j+=1;	
          whichSCLns(j)=i;    	
	  if(surveynosexCL_LikeType(i) > 0) 
	     {
	       jj+=1;
	       whichLikeSCLns(jj)=j;
	     }  
          nObsSCLns(j)=howmanytmpSCLns(2,i);
        }    
     }

 END_CALCS

  matrix yearSCLns(1,howManySCLns,1,nObsSCLns) 
  3darray ObsSCLns(1,howManySCLns,1,nObsSCLns,1,iPoolLength*Nsexes)
  matrix ObsSsizeSCLns(1,howManySCLns,1,nObsSCLns)

 LOCAL_CALCS
  ivector indexposSCLns(1,howManySCLns);
  itmp=0;
  indexposSCLns.initialize();
  for (i=1;i<=NsurveynosexCL;i++)  
     {
        indexx = int(surveynosexCL(i,1));  
        indexx = howmanytmpSCLns(3,indexx);
        indexposSCLns(indexx)+=1;
        itmp=indexposSCLns(indexx);
        yearSCLns(indexx,itmp)=surveynosexCL(i,2);
        ObsSsizeSCLns(indexx,itmp)=surveynosexCL(i,3); 
  
         for (int sex=1;sex<=Nsexes; sex++)
            {
              for (int length=1; length<iPoolLength;length++)
                {
                  ObsSCLns(indexx,itmp,iPoolLength*(sex-1)+length)=surveynosexCL(i,Nlens*(sex-1)+length+3);
      
                 }
              for (length=iPoolLength; length<=Nlens;length++)
                 {
                    ObsSCLns(indexx,itmp,iPoolLength*sex)+=surveynosexCL(i,Nlens*(sex-1)+length+3);
                 }
            }
       }
 END_CALCS

//------------------------------

!!if (debug==1){char mark; cout<<"Nqsurveys= "<<Nqsurveys<<endl; cin>>mark; }

  number pi
  3darray w(StartYear,EndYear+1,1,Nsexes,1,Nages)
  matrix fec(StartYear,EndYear+1,1,Nages)
  number lb
  number ub
  number nagestemp
  number tautemp
  int phz
  
  int i
  int decide
  int yr1CPUE       
  int yr2CPUE
  int yr1
  int yr2
  ivector yr1CA(1,Nmethods)  
  ivector yr2CA(1,Nmethods)  
  matrix Lmean(1,Nsexes,1,Nages)
  vector tempprior(1,7)
  number initialsigma
  number annualsigma
  number biasRannual
  number biasRinitial
  3darray LgivenA(1,Nsexes,1,Nages,1,Nlens)
  matrix Lsd(1,Nsexes,1,Nages)
  number lensum
  vector real_length(1,Nlens)
  matrix real_weight(1,Nsexes,1,Nlens)
  
  matrix CAobtemp(1,NCA,1,PoolAge*Nsexes)
  matrix CAsurveyobtemp(1,NsurveyCA,1,PoolAge*Nsexes)
  matrix CLobtemp(1,NCL,1,iPoolLength*Nsexes)
  matrix CLsurveyobtemp(1,NsurveyCL,1,iPoolLength*Nsexes)
  matrix CLsurveynosexobtemp(1,NsurveynosexCL,1,iPoolLength)
  matrix Apool(1,Nages,1,PoolAge)
  matrix AerrorApool(1,Nages,1,PoolAge)
 
  ivector ystart(1,NCPUEindex)
  ivector yend(1,NCPUEindex)
  ivector ystartCA(1,howManyCA)
  ivector yendCA(1,howManyCA)

 //flag for the report section : New
  ivector report_flag(1,10)
 //flag for the report section : New

 LOCAL_CALCS
   rec_seed=1211;
   assessment_seed=8811;
 END_CALCS

 !!cout<<"End of Data Section"<<endl;
//============================================================================
PARAMETER_SECTION
 LOCAL_CALCS
   phz=R0_prior(1);
   lb=(R0_prior(2));
   ub=(R0_prior(3));
 END_CALCS
  init_bounded_number R0(lb,ub,phz)
  
 LOCAL_CALCS
   phz=h_prior(1);
   lb=h_prior(2);
   ub=h_prior(3);
 END_CALCS
  init_bounded_number h(lb,ub,phz)

 LOCAL_CALCS
  ivector vphz(1,Nsexes);
  dvector vlb(1,Nsexes);
  dvector vub(1,Nsexes);
  for (int sex=1;sex<=Nsexes; sex++)
    {  
      vphz(sex)= M_prior(sex,1);
      vlb(sex) = M_prior(sex,2);
      vub(sex) = M_prior(sex,3);
    }
 END_CALCS
  init_bounded_number_vector M(1,Nsexes,vlb,vub,vphz)

 LOCAL_CALCS
   phz=p_log_InitialDev(1);
   lb=p_log_InitialDev(2);
   ub=p_log_InitialDev(3);
   initialsigma=p_log_InitialDev(6);
 END_CALCS
  init_bounded_vector log_InitialDev(2,Nages-1,lb,ub,phz)

 LOCAL_CALCS
   phz=p_log_RecDev(1);
   lb=p_log_RecDev(2);
   ub=p_log_RecDev(3);
   annualsigma=p_log_RecDev(6);
 END_CALCS
  init_bounded_vector log_RecDev(StartYear,EndYear,lb,ub,phz)

 LOCAL_CALCS
   phz=Rinit_prior(1);
   lb=Rinit_prior(2);
   ub=Rinit_prior(3);
 END_CALCS
  init_bounded_number Rinit(lb,ub,phz)

 LOCAL_CALCS
  for (sex=1;sex<=Nsexes; sex++)
    {  
      vphz(sex)= uinit_prior(sex,1);
      vlb(sex) = uinit_prior(sex,2);
      vub(sex) = uinit_prior(sex,3);
    }
 END_CALCS
  init_bounded_number_vector uinit(1,Nsexes,vlb,vub,vphz)

 LOCAL_CALCS
  for (sex=1;sex<=Nsexes; sex++)
    {  
      vphz(sex)= p_plusscale(sex,1);
      vlb(sex) = p_plusscale(sex,2);
      vub(sex) = p_plusscale(sex,3);
    }
 END_CALCS
  init_bounded_number_vector plusscale(1,Nsexes,vlb,vub,vphz)

 LOCAL_CALCS
  ivector fphz(1,Nmethods);
  dvector flb(1,Nmethods);
  dvector fub(1,Nmethods);
  for (int method=1;method<=Nmethods; method++)
    {  
      fphz(method)= p_Sfullest(method,1);
      flb(method) = p_Sfullest(method,2);
      fub(method) = p_Sfullest(method,3);
    }
 END_CALCS
  init_bounded_number_vector Sfullest(1,Nmethods,flb,fub,fphz) //for length

 LOCAL_CALCS
  for (method=1;method<=Nmethods; method++)
    {  
      fphz(method)= p_Sfulldelta(method,1);
      flb(method) = p_Sfulldelta(method,2);
      fub(method) = p_Sfulldelta(method,3);
    }

 END_CALCS
  init_bounded_number_vector Sfulldelta(1,Nmethods,flb,fub,fphz) //for length

 LOCAL_CALCS
  for (method=1;method<=Nmethods; method++)
    {  
      fphz(method)= log_varLest_prior(method,1);
      flb(method) = log_varLest_prior(method,2);
      fub(method) = log_varLest_prior(method,3);
    }
 END_CALCS
  init_bounded_number_vector log_varLest(1,Nmethods,flb,fub,fphz) //for both

 LOCAL_CALCS
  for (method=1;method<=Nmethods; method++)
    {  
      fphz(method)= log_varRest_prior(method,1);
      flb(method) = log_varRest_prior(method,2);
      fub(method) = log_varRest_prior(method,3);
    }
 END_CALCS
  init_bounded_number_vector log_varRest(1,Nmethods,flb,fub,fphz) //for both


 LOCAL_CALCS

  ivector qphzCA(1,Nmethods);
  dvector qlbCA(1,Nmethods);
  dvector qubCA(1,Nmethods);
  for (method=1;method<=Nmethods; method++)
    {  
      qphzCA(method)= errSfull_prior(method,1);
      qlbCA(method) = errSfull_prior(method,2);
      qubCA(method) = errSfull_prior(method,3);
    }

  for (method=1;method<=howManyCA; method++)
    {  
      ystartCA(method) = yearCA(method,1)+1;
      yendCA(method)   = yearCA(method,nObsCA(method)) ; 

    }
   
 END_CALCS
  init_bounded_vector_vector errSfull(1,Nmethods,ystartCA,yendCA,qlbCA,qubCA,qphzCA) 

 LOCAL_CALCS
  for (method=1;method<=Nmethods; method++)
    {  
      qphzCA(method)= errvarL_prior(method,1);
      qlbCA(method) = errvarL_prior(method,2);
      qubCA(method) = errvarL_prior(method,3);
    }

 END_CALCS
  init_bounded_vector_vector errvarL(1,Nmethods,ystartCA,yendCA,qlbCA,qubCA,qphzCA) 
  
 LOCAL_CALCS
  for (method=1;method<=Nmethods; method++)
    {  
      qphzCA(method)= errvarR_prior(method,1);
      qlbCA(method) = errvarR_prior(method,2);
      qubCA(method) = errvarR_prior(method,3);
    }
 END_CALCS
  init_bounded_vector_vector errvarR(1,Nmethods,ystartCA,yendCA,qlbCA,qubCA,qphzCA) 

 LOCAL_CALCS
  ivector qphz(1,NCPUEindex);
  dvector qlb(1,NCPUEindex);
  dvector qub(1,NCPUEindex);
  for (method=1;method<=NCPUEindex; method++)
    {  
      qphz(method)= log_qCPUE_prior(method,1);
      qlb(method) = log_qCPUE_prior(method,2);
      qub(method) = log_qCPUE_prior(method,3);
    }
 END_CALCS
  init_bounded_number_vector log_qCPUE(1,NCPUEindex,qlb,qub,qphz)

 LOCAL_CALCS
  for (method=1;method<=NCPUEindex; method++)
    {  
      qphz(method)= qCPUEerr_prior(method,1);
      qlb(method) = qCPUEerr_prior(method,2);
      qub(method) = qCPUEerr_prior(method,3);
      ystart(method) = yearCPUE(method,1)+1;
      yend(method)   = yearCPUE(method,nObsCPUE(method)) ; 
    }
 END_CALCS
  init_bounded_vector_vector qCPUEerr(1,NCPUEindex,ystart,yend,qlb,qub,qphz) 

 LOCAL_CALCS
  ivector sphz(1,Nqsurveys);
  dvector slb(1,Nqsurveys);
  dvector sub(1,Nqsurveys);

  for (method=1;method<=Nqsurveys; method++)
    {  
      sphz(method)= log_qsurvey_prior(method,1);
      slb(method) = log_qsurvey_prior(method,2);
      sub(method) = log_qsurvey_prior(method,3);
    }
 END_CALCS
  init_bounded_number_vector log_qsurvey(1,Nqsurveys,slb,sub,sphz)

 LOCAL_CALCS
  for (method=1;method<=Nqsurveys; method++)
    {  
      sphz(method)= surveySfull_prior(method,1);
      slb(method) = surveySfull_prior(method,2);
      sub(method) = surveySfull_prior(method,3);
    } 
 END_CALCS
  init_bounded_number_vector surveySfullest(1,Nqsurveys,slb,sub,sphz)

 LOCAL_CALCS
   for (method=1;method<=Nqsurveys; method++)
    {  
      sphz(method)= p_surveySfulldelta(method,1);
      slb(method) = p_surveySfulldelta(method,2);
      sub(method) = p_surveySfulldelta(method,3);
    } 
 
 END_CALCS

  init_bounded_number_vector surveySfulldeltaest(1,Nqsurveys,slb,sub,sphz) //for length

 LOCAL_CALCS
  for (method=1;method<=Nqsurveys; method++)
    {  
      sphz(method)= log_surveyvarL_prior(method,1);
      slb(method) = log_surveyvarL_prior(method,2);
      sub(method) = log_surveyvarL_prior(method,3);
    } 
 END_CALCS

  init_bounded_number_vector log_surveyvarL(1,Nqsurveys,slb,sub,sphz)

 LOCAL_CALCS
  for (method=1;method<=Nqsurveys; method++)
    {  
      sphz(method)= log_surveyvarR_prior(method,1);
      slb(method) = log_surveyvarR_prior(method,2);
      sub(method) = log_surveyvarR_prior(method,3);
    } 
 END_CALCS
  init_bounded_number_vector log_surveyvarR(1,Nqsurveys,slb,sub,sphz)

 LOCAL_CALCS
   phz=tempfind_prior(1);
   lb=tempfind_prior(2);
   ub=tempfind_prior(3);
 END_CALCS
  init_number tempfind(phz)  // Dummy vector

  number varL
  number varR
  number Sfull
  3darray N(StartYear,EndYear+1,1,Nsexes,1,Nages)
  3darray Bio(StartYear,EndYear+1,1,Nsexes,1,Nages)
  4darray va(1,Nmethods,StartYear,EndYear+1,1,Nsexes,1,Nages)
  3darray survey_w(1,Nsurveyindex,1,Nsexes,1,Nages)
  3darray surveyva(1,Nsurveyindex,1,Nsexes,1,Nages)
  3darray surveyvl(1,Nsurveyindex,1,Nsexes,1,Nlens)
  matrix B(1,Nmethods,StartYear,EndYear+1) //mid year biomass
  vector S(StartYear,EndYear+1) //start of year
  matrix u(1,Nmethods,StartYear,EndYear)
  number alpha
  number beta
  number S0

  number VVB 
  number decaysum
  number new_R0
  vector decay(1,Nages)  

  3darray CAp(1,NCA,1,Nsexes,1,PoolAge)
  3darray nCAp(1,howManyCA,1,nObsCA,1,PoolAge*Nsexes)
  3darray surveyCAp(1,NsurveyCA,1,Nsexes,1,PoolAge)
  matrix CApredtemp(1,NCA,1,PoolAge*Nsexes)
  3darray nsurveyCAp(1,howManySCA,1,nObsSCA,1,PoolAge*Nsexes)

  matrix surveyCApredtemp(1,NsurveyCA,1,PoolAge*Nsexes)
  matrix CLp(1,NCL,1,Nsexes*iPoolLength)
  3darray nCLp(1,howManyCL,1,nObsCL,1,iPoolLength*Nsexes)

  matrix surveyCLp(1,NsurveyCL,1,Nsexes*iPoolLength)
  3darray nsurveyCLp(1,howManySCL,1,nObsSCL,1,iPoolLength*Nsexes)

 
  matrix surveynosexCLp(1,NsurveynosexCL,1,iPoolLength)
  3darray nsurveynosexCLp(1,howManySCLns,1,nObsSCLns,1,iPoolLength)
  vector CLpredtemp(1,iPoolLength*Nsexes)
  vector predCPUE(1,NCPUE)

  vector npredCPUE(1,NCPUE)
  
  matrix qCPUE(1,NCPUEindex,StartYear,EndYear+1)
  vector predsurvey(1,Nsurvey)
  vector qsurvey(1,Nsurveyindex)
  vector surveyvarL(1,Nsurveyindex)
  vector surveyvarR(1,Nsurveyindex)
  vector surveySfull(1,Nsurveyindex)
  vector surveySfulldelta(1,Nsurveyindex)

  number utotal
  number utotal2
  number utotalbefore
  number afterutotal
  
  vector surv(1,Nsexes);
  vector surv05(1,Nsexes);
  matrix vinit(1,Nsexes,1,Nages) 
  
  objective_function_value f

// Arrays used in projection section
  vector Bproject(EndYear+1,ProjectYear+1)
  !!int indextemp=(EndStrategy-StartStrategy)/StepStrategy+2;
  matrix Cproject(1,indextemp,EndYear+1,ProjectYear+1)
  matrix Sproject(1,indextemp,EndYear+1,ProjectYear+1)
  3darray Nproject(EndYear+1,ProjectYear+1,1,Nsexes,1,Nages)
  matrix strategyva(1,Nsexes,1,Nages)
  matrix wproject(1,Nsexes,1,Nages)
  vector fecproject(1,Nages)
  vector randrec(1,1)
  vector randhr(1,1)

// likelihood arrays
  vector CPUELL(1,NCPUEindex)
  vector SurveyLL(1,Nsurveyindex)
  vector CALL(1,Nmethods)
  vector CLLL(1,Nmethods)
  vector surveyCALL(1,Nsurveyindex)
  vector surveynosexCLLL(1,Nsurveyindex)
  vector surveyCLLL(1,Nsurveyindex);

//  vector nCPUELL(1,NCPUEindex)
  vector nCALL(1,howManyCA) //Nmethods)
  vector nCLLL(1,howManyCL)
  vector nsurveyCALL(1,howManySCA)  //nUsedSCA
  vector nsurveynosexCLLL(1,howManySCLns)
  vector nsurveyCLLL(1,howManySCL)

// Accumulated Prior value
  number Priorlike

// Penalty for exploitation rate
  number negpen


// in order to run MCMC
  sdreport_number Ro_mcmc

//=============================================================================================================================================
PRELIMINARY_CALCS_SECTION
//***********************

 cout<<"   Running Model.........."<<endl;
 //assign paramters with their initial values from the prior vectors in the .dat file

  if(debug!=2) // Use *.txt or colera30.par as input file for parameters
    {  
      initialize_parameters();
    }
 
   for (i=1;i<=Nsurveyindex;i++)
      {
         qsurvey(i) = mfexp(log_qsurvey(i)); 
         surveyvarL(i) = mfexp(log_surveyvarL(i)); 
         surveyvarR(i) = mfexp(log_surveyvarR(i)); 
         surveySfull(i) = surveySfull(i); 
      }


  extract_data();
  setup_constant_parameters();
  if(useBound<=0 || useBound>= 1.0) useBound=0.99;
  decide=0;
//=============================================================================================================================================
PROCEDURE_SECTION
//***************
   nsurveyCALL.initialize();				// Reset partial objective functions 
   nsurveyCLLL.initialize();
   nsurveynosexCLLL.initialize();
   CPUELL.initialize();
   nCALL.initialize();
   nCLLL.initialize();
   SurveyLL.initialize();
   f=0; 						// Reset total objective function 
   negpen=0.0;  					// Reset penalty for the exploitation rate

   setup_variable_parameters(); 			// Core of the model ....
   initial_conditions(); 
   dynamics();

  if(sum(zeroSIndextmp) > 0) nSurveyLike(); 		// Survey Index Likelihood
     f+=sum(elem_prod(SurveyLL,zeroSIndextmp));

  if(sum(zeroCPUEtmp) > 0) nCPUELike();			// CPUE Likelihood
     f+=sum(elem_prod(CPUELL,zeroCPUEtmp));


  if(sum(zeroSCAtmp) > 0) nsurveyCALike(); 		// Survey CA Likelihood
     f+=sum(elem_prod(nsurveyCALL,zeroSCAtmp));

  if(sum(zeroSCLtmp) > 0) nsurveyCLLike(); 		// Survey CL Likelihood
    f+=sum(elem_prod(nsurveyCLLL , zeroSCLtmp));

  if(sum(zeroSCLtmpns) > 0) nsurveynosexCLLike(); 	// Survey no sex CL Likelihood
    f+=sum(elem_prod(nsurveynosexCLLL,zeroSCLtmpns));

  if(sum(zeroCAtmp) > 0) nCALike(); 			// Commercial Fleet CA Likelihood
    f+=sum(elem_prod(nCALL,zeroCAtmp));

  if(sum(zeroCLtmp) > 0) nCLLike();			// Commercial Fleet CL Likelihood
    f+=sum(elem_prod(nCLLL , zeroCLtmp));

    f+=negpen; 						// Add Penalty for exploitation rate
    evaluate_priors(); 					// Prior calculation
    f+=Priorlike;   					// Add prior  
    if(mceval_phase()) 					// Labels for projection file
      {
        if(decide==0)
          {
            cout<<"      Evaluation phase  ..........  "<<endl;
            decide = 1;

            the_lables(); 
          }
      }

  if(mceval_phase()) Projections(); 			//Do Projections
   Ro_mcmc=R0;  					// At least one has to be declared for the mcmc calculation

//=============================================================================================================================================
FUNCTION the_lables
   int year;
   int sex;
   int method;
   int age;   
        	    // Lables for the LIKELIHOODS 
		    //*************************** 
		    proj1<<"CPUE "<<"CALL "<<"CLLL "<<"Survey "<<"surveyCALL "<<"surveyCLLL "<<"surveynosexCLLL "<<"negpen "<<"Priorlike "<<"f "<<endl;

		    // Lables for the PARAMETERS 
		    //************************** 
		    proj2<<"R0"<<" "<<"h"<<" "<<"Rinit"<<" ";
		    for (sex=1;sex<=Nsexes;sex++)
		       {  
			  proj2<<"M_"<<sex<<" ";
		       }
		    for (sex=1;sex<=Nsexes;sex++)
		       { 	 
			  proj2<<"uinit_"<<sex<<" ";
		       }
		    for (sex=1;sex<=Nsexes;sex++)
		       {  
			  proj2<<"plusscale_"<<sex<<" ";
		       }
		    for (method=1;method<=Nmethods; method++)
		       {  
			  proj2<<"Sfullest_"<<method<<" ";
		       }
		    for (method=1;method<=Nmethods; method++)
		       {  
			  proj2<<"Sfulldelta_"<<method<<" ";
		       }
		    for (method=1;method<=Nmethods; method++)
		       {  
			  proj2<<"log_varLest_"<<method<<" ";
		       }
		    for (method=1;method<=Nmethods; method++)
		       {  
			  proj2<<"log_varRest_"<<method<<" ";
		       }
		    for (method=1;method<=NCPUEindex; method++)
		       {  
			  proj2<<"log_qCPUE_"<<method<<" ";
		       }
		    for (method=1;method<=Nqsurveys; method++)
		       {  
			  proj2<<"log_qsurvey_"<<method<<" ";
		       }
		    for (method=1;method<=Nqsurveys; method++)
		       {  
			  proj2<<"surveySfull_"<<method<<" ";
		       }
		    for (method=1;method<=Nqsurveys; method++)
		       {  
			  proj2<<"surveySfulldeltaest_"<<method<<" ";
		       }
		    for (method=1;method<=Nqsurveys; method++)
		       {  
			  proj2<<"log_surveyvarL_"<<method<<" ";
		       }
		    for (method=1;method<=Nqsurveys; method++)
		       {  
			  proj2<<"log_surveyvarR_"<<method<<" ";
		       }
		    proj2<<endl;
	   
	    // LABLES FOR THE RESIDUALS
       proj3<<" ";       
       for (age=2; age<=Nages-1;age++)
          {
 	    proj3<<"InitAge_"<<age<<" ";
	  }  

       for (year = StartYear; year<= EndYear+1; year++)
          {
            proj3<<"Rec_"<<year<<" ";
	  }

	  
       for (method=1;method<=Nmethods; method++)
          {  
            for (year = ystartCA(method); year<= yendCA(method); year++)
               {
  	          proj3<<"ErrSfull_"<<method<<"_"<<year<<" ";
  	       }  
	  } 
	   
       for (method=1;method<=Nmethods; method++)
          {  
            for (year = ystartCA(method); year<= yendCA(method); year++)
               {
  	          proj3<<"ErrvarL_"<<method<<"_"<<year<<" ";
  	       }  
	  } 

       for (method=1;method<=Nmethods; method++)
          {  
            for (year = ystartCA(method); year<= yendCA(method); year++)
               {
  	          proj3<<"ErrvarR_"<<method<<"_"<<year<<" ";
  	       }  
	  } 

       for (method=1;method<=NCPUEindex; method++)
          {  
            for (year = ystart(method); year<= yend(method); year++)
               {
  	          proj3<<"qCPUEerr_"<<method<<"_"<<year<<" ";
  	       }  
	  } 

        proj3<<endl;
	
	proj4<<" ";
	// Lable for the Spawning Biomass
	for (year=StartYear; year<=EndYear+1; year++) 
           {
             proj4<<"SpBiom_"<<year<<" ";
           }	 
	proj4<<endl;   
  
         proj5<<" ";
        // Lable for the Exploitation rate
        for (method=1;method<=Nmethods; method++)
           {  
   	     for (year=StartYear; year<=EndYear; year++) 
                {
                  proj5<<"u_"<<year<<" ";
		}
           }	 
	proj5<<endl;   
  
        proj6<<" ";   
        // Lable for the vulnerable Biomass
        for (method=1;method<=Nmethods; method++)
           {  
   	     for (year=StartYear; year<=EndYear+1; year++) 
                {
                  proj6<<"vBiom_"<<method<<"_"<<year<<" ";
		}
           }	 
	proj6<<endl;   

        proj7<<" ";
        // Lable for the Recruits
	for (year=StartYear; year<=EndYear+1; year++) 
           {
              proj7<<"Reclut_"<<year<<" ";
           }
	proj7<<endl;   
	   

        // Lable for the virgin Recruitment / Virgin Spawning stock / Virgin vulnerable Biomass
	
	proj8<<"Ro"<<" "<<"S0"<<" "<<"VVB"<<endl;
            
	    
        // Lables for Projected Vulnerable Biomass
        proj9<<"Virgin_Vulnerable_Biomass"<<" "<<"Vulnerable_Biomass_"<<StartYear<<" ";
        proj10<<"Virgin_Spawning_Biomass"<<" "<<"Spawning_Biomass_"<<StartYear<<" ";
        proj9<<" "; 
        proj10<<" "; 
        proj11<<" "; 
        proj12<<" "; 
        proj13<<" "; 
        
	
	int Nstrategy= 1 + (EndStrategy-StartStrategy)/StepStrategy;
        if(StrategyType==2) Nstrategy=Nstrategy+1;

        for (int istrategy=1;istrategy<=Nstrategy;istrategy++)
           {

 	     for (year=EndYear+1; year<=ProjectYear+1;year++)
               proj9<<"Biom_"<<year<<"_"<<istrategy<<" ";
        
 	     for (year=EndYear+1; year<=ProjectYear+1;year++)
               proj10<<"SpBiom_"<<year<<"_"<<istrategy<<" ";

             for (year=EndYear+1; year<=ProjectYear+1;year++)
	       proj11<<"Catch_"<<year<<"_"<<istrategy<<" ";

             for (year=EndYear+1; year<=ProjectYear+1;year++)
	       proj12<<"RecRes_"<<year<<"_"<<istrategy<<" ";
             
	     for (year=EndYear+1; year<=ProjectYear+1;year++)
	       proj13<<"Strategy_"<<year<<"_"<<istrategy<<" ";

  	      proj9<<" ";
  	      proj10<<" ";
	      proj11<<" ";
	      proj12<<" ";
	      proj13<<" ";


           } 

	proj9<<endl;
	proj10<<endl;
	proj11<<endl;
	proj12<<endl;
	proj13<<endl;

   
	    
//=============================================================================================================================================


FUNCTION extract_data 
  int length;
  for (int obs=1;obs<=NCA;obs++)
   {
     for (int sex=1;sex<=Nsexes; sex++)
      {
        for (int age=1; age<=PoolAge;age++)
         {
           CAobtemp(obs,PoolAge*(sex-1)+age)=CA(obs,Nages*(sex-1)+age+3);
         }
        for (age=PoolAge+1; age<=Nages;age++)
         {
           CAobtemp(obs,PoolAge*sex)+=CA(obs,Nages*(sex-1)+age+3);
         }
      }
   }
  
 for (obs=1;obs<=NsurveyCA;obs++)
  {
    for (int sex=1;sex<=Nsexes; sex++)
     {
       for (int age=1; age<=PoolAge;age++)
        {
           CAsurveyobtemp(obs,PoolAge*(sex-1)+age)=surveyCA(obs,Nages*(sex-1)+age+3);
        }
       for (age=PoolAge+1; age<=Nages;age++)
        {
           CAsurveyobtemp(obs,PoolAge*sex)+=surveyCA(obs,Nages*(sex-1)+age+3);
        }
     }
  }
  for (obs=1;obs<=NCL;obs++)
     {  
       for (int sex=1;sex<=Nsexes; sex++)
          {
            for (length=1; length<iPoolLength;length++)
               {
                  CLobtemp(obs,iPoolLength*(sex-1)+length)=CL(obs,Nlens*(sex-1)+length+3);
               }
      
            for (length=iPoolLength; length<=Nlens;length++)
               {
                  CLobtemp(obs,iPoolLength*sex)+=CL(obs,Nlens*(sex-1)+length+3);
               }
          }
     }
  for (obs=1;obs<=NsurveyCL;obs++)
     {
       for (int sex=1;sex<=Nsexes; sex++)
          {
            for (length=1; length<iPoolLength;length++)
               {
                  CLsurveyobtemp(obs,iPoolLength*(sex-1)+length)=surveyCL(obs,Nlens*(sex-1)+length+3);
               }
            for (length=iPoolLength; length<=Nlens;length++)
               {
                  CLsurveyobtemp(obs,iPoolLength*sex)+=surveyCL(obs,Nlens*(sex-1)+length+3);
               }
          }
     }
  for (obs=1;obs<=NsurveynosexCL;obs++)
     {
       for (length=1; length<iPoolLength;length++)
          {
             CLsurveynosexobtemp(obs,length)=surveynosexCL(obs,length+3);
          }
       for (length=iPoolLength;length<=Nlens;length++)
          {
             CLsurveynosexobtemp(obs,iPoolLength)+=surveynosexCL(obs,length+3);
          }
     }


//=============================================================================================================================================
FUNCTION initialize_parameters
// *****************************
  R0=R0_prior(7); 
  h=h_prior(7); 
  log_InitialDev=p_log_InitialDev(7);
  log_RecDev=p_log_RecDev(7);
  Rinit=Rinit_prior(7);


  
  for (int sex=1;sex<=Nsexes;sex++)
    {
      M(sex)=M_prior(sex,7);
      uinit(sex)=uinit_prior(sex,7);
      plusscale(sex)=p_plusscale(sex,7);
    }
// FISHERY SELECTIVITY PARAMETERS

  for (int method=1;method<=Nmethods;method++)
    {
      Sfullest(method)=p_Sfullest(method,7); //for length
      Sfulldelta(method)=p_Sfulldelta(method,7); //for length
      log_varLest(method)=log_varLest_prior(method,7); 
      log_varRest(method)=log_varRest_prior(method,7); 
      errSfull(method)=errSfull_prior(method,7);
      errvarL(method)=errvarL_prior(method,7);
      errvarR(method)=errvarR_prior(method,7);

    }
// FISHERY CATCHABILITY PARAMETERS  
  for (int index=1;index<=NCPUEindex;index++)
     {
      log_qCPUE(index)=log_qCPUE_prior(index,7);

      for (int year=ystart(index);year<=yend(index); year++)
         {
            qCPUEerr(index,year)= qCPUEerr_prior(index,7);


         } 
     }
// SURVEY SELECTIVITY-CATCHABILITY PARAMETERS
  for (index=1;index<=Nqsurveys;index++)
    {     
      log_surveyvarL(index)=log_surveyvarL_prior(index,7);  
      log_surveyvarR(index)=log_surveyvarR_prior(index,7);  
      surveySfullest(index)=surveySfull_prior(index,7); 
      surveySfulldeltaest(index)=p_surveySfulldelta(index,7); 
      log_qsurvey(index)=log_qsurvey_prior(index,7);  
    }
//=============================================================================================================================================
FUNCTION setup_constant_parameters
// *******************************
  Apool.initialize();
  int sex;
  int age;
  int age1;
  int year;
  int obs;
  int length;
  
  for (age1=1; age1<=PoolAge;age1++)
     {
        Apool(age1,age1)=1.0;
     }
  for (age1=PoolAge+1;age1<=Nages;age1++)
     {
        Apool(age1,PoolAge)=1.0;
     }

  AerrorApool=Aerror*Apool;

//Length 
//OjO-cambio 1/24/2002
  fec.initialize();

  for (sex=1;sex<=Nsexes;sex++)
   {
     for(age=1;age<=Nages;age++)
      {
        Lmean(sex,age)=linf(sex)*(1.-mfexp(-k(sex)*(age-t0(sex)) ));
      }
   }

  for (sex=1;sex<=Nsexes;sex++)
    {
     switch (type_sd_rel(1))
       {
        case 1: // linear relationship with age
          for (age=1;age<=Nages;age++)
            {
              Lsd(sex,age)=Lsd1(sex)+((Lsdn(sex)-Lsd1(sex))/(Nages-1))*(age - 1);
            }
          break;
        case 2:  // linear relationship with length 
          for (age=1;age<=Nages;age++)
            {
     	        Lsd(sex,age)=Lsd1(sex)+((Lsdn(sex)-Lsd1(sex))/(Lmean(sex,Nages)-Lmean(sex,1)))*(Lmean(sex,age)-Lmean(sex,1));            
            }
          break;
        case 3: // mix :  
          for (age=1;age<=Nages;age++)
            {
	      Lsd(sex,age)=Lsd1(sex)+(Lsdn(sex)-Lsd1(sex))*(1.0 - pow(exp(-1.0 * k(sex)), age - 1))/(1.0 - pow(exp(-1.0 * k(sex)),Nages - 1));
            }
          break;
	  default:
	    cout<<" Not such Lsd relationship , Options --> 1: Age / 2: Length / 3: mix "<<endl;
	  break;    
       
       } 
       Lsd(sex)=elem_div(Lsd(sex),Lmean(sex)); //for lognormal
 
    }       

//Weight and fecundity
// que pasa con el WA? 
 
  fec.initialize();
  for (year=StartYear;year<=EndYear+1;year++)
    {
     for (age=1;age<=Nages;age++)
       {
        for (sex=1;sex<=Nsexes;sex++)
          {
            w(year,sex,age)=bi(sex)*pow(Lmean(sex,age),bii(sex))*exp(0.5*bii(sex)*square(Lsd(sex,age))*(bii(sex)-1.0)); 
          }
            fec(year,age)=w(year,1,age)*mat(age);
       }       
    }
   
  if (useWA==1)
    {
      for (obs=1;obs<=NWA;obs++)
        {
           year=WA(obs,1);
           sex=WA(obs,2);
           for (age=1;age<=Nages;age++)
             {
               w(year,sex,age)=WA(obs,age+2);
               fec(year,age)=w(year,1,age)*mat(age);
             }
        }
   
    }
 
    
//length stuff
   for (length=1;length<=Nlens;length++)
      {
        real_length(length)=(length-1)*IncL+FirstL+IncL*0.5;
      }
      
      
   for (sex=1;sex<=Nsexes;sex++)
      {
        real_weight(sex)=bi(sex)*pow(real_length,bii(sex));
        LgivenA(sex) = Calc_LgivenA (Nages,Nlens,real_length,Lmean(sex),Lsd(sex),IncL);
      }   

//=============================================================================================================================================
FUNCTION setup_variable_parameters
// survival
  int method;
  int sex; 
  int year;
  int age;
  int survey;
  int index;
  int ii;
   
  for (sex=1;sex<=Nsexes;sex++)
    {
      surv(sex)=exp(-M(sex));
      surv05(sex)=exp(-0.5*M(sex));
    }

// bias correction if Recruitment and Initial Age structure RESIDUALS are used.
  biasRannual=exp(-0.5*square(annualsigma));
  biasRinitial=exp(-0.5*square(initialsigma));
  
  for (index=1;index<=Nqsurveys;index++)
    {     
      qsurvey(index)=exp(log_qsurvey(index));
      surveyvarL(index)=exp(log_surveyvarL(index));
      surveyvarR(index)=exp(log_surveyvarR(index));
      surveySfull(index)=surveySfullest(index);
      surveySfulldelta(index)=surveySfulldeltaest(index);
    }
// AGE Specific commercial selectivity
  dvar_matrix Sfullv(1,Nmethods,StartYear,EndYear+1); 
  dvar_matrix varLv(1,Nmethods,StartYear,EndYear+1);
  dvar_matrix varRv(1,Nmethods,StartYear,EndYear+1);
  va.initialize();
  Sfullv.initialize();
  varLv.initialize();
  varRv.initialize();
// IF no residuals are estimated, then

  for(method=1;method<=Nmethods;method++)
    {

      varR  = exp(log_varRest(method));
      varL  = exp(log_varLest(method));
  
      Sfullv(method) = Sfullest(method); 
      varRv(method)  = exp(log_varRest(method));
      varLv(method)  = exp(log_varLest(method));
    } 
// BUT there might be trends for some methods
   for(method=1;method<=howManyCA;method++)
     {
       ii=whichCA(method);      
      if (active(errSfull(ii))) 
        {
          for (year=ystartCA(method);year<=yendCA(method);year++)
             Sfullv(ii,year)=Sfullv(ii,year-1)*exp(errSfull(ii,year));

          for (year=yendCA(method)+1;year<=EndYear+1;year++)
             Sfullv(ii,year)=Sfullv(ii,yendCA(method));          
        }

      if (active(errvarR(ii))) 
        {
          for (year=ystartCA(method);year<=yendCA(method);year++)
             varRv(ii,year)=varRv(ii,year-1)*exp(errvarR(ii,year));

          for (year=yendCA(method)+1;year<=EndYear+1;year++)
             varRv(ii,year)=varRv(ii,yendCA(method));          
        } 
      if (active(errvarL(ii))) 
        {
          for (year=ystartCA(method);year<=yendCA(method);year++)
             varLv(ii,year)=varLv(ii,year-1)*exp(errvarL(ii,year));

          for (year=yendCA(method)+1;year<=EndYear+1;year++)
             varLv(ii,year)=varLv(ii,yendCA(method));          
        } 
      } // method
  
  
  for(method=1;method<=Nmethods;method++)
    {
     for (sex=1;sex<=Nsexes;sex++)
       {
         for(year=StartYear;year<=EndYear+1;year++)
            {
             //else stay with last values
             for (age=1;age<=Nages;age++)
               {
                 if ((Sfullv(method,year)+ (sex-1) * Sfulldelta(method))>age)
                   {
                     va(method,year,sex,age)=exp(-1.0*(square(age-(Sfullv(method,year)+ (sex-1) * Sfulldelta(method))))/varLv(method,year));             
                   }
                 else
                   {
                     va(method,year,sex,age)=exp(-1.0*(square(age-(Sfullv(method,year)+ (sex-1) * Sfulldelta(method))))/varRv(method,year));             
                   }
               }//age
             }//year
         }//sex 
    }//method

//SURVEY Age Specific Selectivity
  dvariable Sfulltemp;
  for(survey=1;survey<=Nsurveyindex;survey++)
    {
      if (survey_vType(survey)==1) //this is age based
        {
         for (sex=1;sex<=Nsexes;sex++)
           {
             Sfulltemp = surveySfull(survey) + (sex-1) * surveySfulldelta(survey);
             surveyva(survey,sex)=selage( Sfulltemp, surveyvarL(survey), surveyvarR(survey), Nages);         
           }
        }
      else if (survey_vType(survey)==2) //this is length based
       {
         for (sex=1;sex<=Nsexes;sex++)
           {
              Sfulltemp = surveySfull(survey) + (sex-1) * surveySfulldelta(survey);
              surveyvl(survey,sex)=sellen( Sfulltemp, surveyvarL(survey), surveyvarR(survey), real_length, Nlens);         
              // now integrate age-specific selectivity             
              for(int age=1;age<=Nages;age++)
                {
                  dvar_vector temp =elem_prod(surveyvl(survey,sex),LgivenA(sex,age));
                  surveyva(survey,sex,age)=sum(temp);                          
                  survey_w(survey,sex,age)=(real_weight(sex)*temp)/surveyva(survey,sex,age);
                }
           } //sex
       }  //if size based
    }//survey

  for (index=1;index<=NCPUEindex;index++)
     {     
      qCPUE(index)=exp(log_qCPUE(index));
      // but qCPUEerr is only estimated for years when there is information
      for (year=ystart(index);year<=yend(index);year++)
         {
          qCPUE(index,year)=qCPUE(index,year-1)*exp(qCPUEerr(index,year)); //error
         }
      for (year=yend(index)+1;year<=EndYear+1;year++)
         {
          qCPUE(index,year)=qCPUE(index,yend(index));
         }
     }
//Selectivity vector for the historic exploitation rate 
  if (ini_Gear==0) vinit=vinit_in;
  else vinit = va(ini_Gear,StartYear);

//=============================================================================================================================================
FUNCTION initial_conditions
  int age;
  int sex;
  int method;
  
  N.initialize();
  B.initialize();

  
//Virgin Spawning Biomass
   S0=0;
   for (age=1;age<=Nages-1;age++)
     {
       S0+=R0 * RecFraction(1) * exp((age-1) * -1.0 * M(1)) * fec(StartYear,age);  //Virgin bimoass is when weight is same as start year
     }

   S0+=((R0 * RecFraction(1) * exp((Nages-1) * -1.0 * M(1)))/(1-exp(-1.0 * M(1))))*fec(StartYear,Nages);

//Compute Recruitment Parameters

   alpha=(S0/R0)*(1-((h-0.2)/(0.8*h)));
   beta=(5*h-1)/(4*h*R0);


   new_R0=0.0; 
   decaysum=0.0;  
   decay=0.0;

//   Now Compute New R0
//   decay(1)=1.0;
//   for (age=2;age<Nages;age++)
//      {
//        decay(age) = decay(age-1)*(1.0-vinit(1,age-1)*uinit(1))*exp(-1.0 * M(1)); 
//      }
//   decay(Nages) = decay(Nages-1)*(exp(-1.0 * M(1))*(1.0-vinit(1,Nages-1)*uinit(1))) / (1.- exp(-1.0 * M(1))*(1.0-vinit(1,Nages-1)*uinit(1)));    	
//   for (age=1;age<=Nages;age++)
//     {
//         decaysum += decay(age) * fec(StartYear,age);
//     }
//   new_R0 = (decaysum - alpha)/(decaysum * beta);

    new_R0=R0; 
  
  for (sex=1;sex<=Nsexes;sex++)
    {
  
     
     N(StartYear,sex,1)=Rinit * new_R0 * RecFraction(sex);
  
  
     
     //Initial deterministic age-structure
     for (age=1;age<=Nages-1;age++)
       {
          N(StartYear,sex,age+1)=N(StartYear,sex,age)*surv(sex)*(1-uinit(sex)*vinit(sex,age));
       }
     //Correction for Plus group
     N(StartYear,sex,Nages)/=(1-surv(sex)*(1-uinit(sex)*vinit(sex,Nages)));
     
     //Add initial age-structure residuals
     if(active(log_RecDev))
       {   
         N(StartYear,sex,1)*=exp(log_RecDev(StartYear))*biasRannual;
       }
     else
       {
         N(StartYear,sex,1)*=exp(log_RecDev(StartYear));
       }
     for (age=2;age<=Nages-1;age++)
       {
         if(active(log_InitialDev))
           {
             N(StartYear,sex,age)*=exp(log_InitialDev(age))*biasRinitial; 
           }
         else
           {
             N(StartYear,sex,age)*=exp(log_InitialDev(age)); 
           }
       }
   
     N(StartYear,sex,Nages)*=plusscale(sex);

     //Initial Vulnerable biomass by method
     for (method=1; method<=Nmethods;method++)
       {
         B(method,StartYear)+=(elem_prod(N(StartYear,sex),w(StartYear,sex))*va(method,StartYear,sex))*surv05(sex);
       }
   }//sex
   
   //Initial Spawning biomass
   S(StartYear)=N(StartYear,1)*fec(StartYear);


//Virgin vulnerable Biomass
   VVB=0;
   for (sex=1;sex<=Nsexes;sex++)
     {
       for (age=1;age<=Nages-1;age++)
         {
           VVB+=R0 * RecFraction(sex) * exp((age-1) * -1.0 * M(sex)) * va(1,StartYear,sex,age) * w(StartYear,sex,age);  
         }
       VVB+=((R0 * RecFraction(sex) * exp((Nages-1) * -1.0 * M(sex)))/(1 - exp(-1.0 * M(sex)))) * va(1,StartYear,sex,Nages) * w(StartYear,sex,Nages);
     }

//=============================================================================================================================================

FUNCTION dynamics //sex=1 is Female
  
  for (int year=StartYear;year<=EndYear-1;year++)
    {
      utotal=0;

      for (int method=1;method<=Nmethods;method++)
        {
          u(method,year)=C(method,year)/(B(method,year));
          utotal+=u(method,year);
        }
      utotalbefore=utotal;
      if(utotal>=useBound) 
	 {       
           utotal=useBound;
           negpen+= 1000 * square(useBound-utotalbefore);    
	 }

      for (int sex=1;sex<=Nsexes;sex++)
        {

          for (int age=1;age<=Nages-1;age++) //exploitation
            {
              afterutotal=0.0;	
              for (int method=1;method<=Nmethods;method++)
                 {
                   afterutotal+=u(method,year)*va(method,year,sex,age)*utotal/utotalbefore;
                 }
              N(year+1,sex,age+1)=N(year,sex,age)*(1.0-afterutotal);
            }

            afterutotal=0;	
            for (int method=1;method<=Nmethods;method++) //Max Age
               {
                 afterutotal+=u(method,year)*va(method,year,sex,Nages)*utotal/utotalbefore;               
               }
            N(year+1,sex,Nages)+=N(year,sex,Nages)*(1.0-afterutotal);

            //natural mortality
            N(year+1,sex)*=surv(sex);
              //recruitment
            if(active(log_RecDev))
              {
                N(year+1,sex,1)=(S(year)/(alpha+beta*S(year)))*exp(log_RecDev(year+1))*biasRannual*RecFraction(sex);
              }
              else
              {
                N(year+1,sex,1)=(S(year)/(alpha+beta*S(year)))*exp(log_RecDev(year+1))*RecFraction(sex); 

              }
                 //Vulnerable biomass by method
               for (method=1;method<=Nmethods;method++)
                 {
                   B(method,year+1)+=(elem_prod(N(year+1,sex),w(year+1,sex))*va(method,year+1,sex))*surv05(sex);
                 }
          }//sex
        //Spawning biomass
        S(year+1)=N(year+1,1)*fec(year+1);
     }//year
     
// for the last year + 1
// =====================
     for (int method=1;method<=Nmethods;method++)
        {
          u(method,EndYear)=C(method,EndYear)/(B(method,EndYear));
          utotal+=u(method,EndYear);
        }
      utotalbefore=utotal;
      if(utotal>=useBound) 
	 {       
           utotal=useBound;
           negpen+=1000*square(useBound-utotalbefore);    
	 }

      for (int sex=1;sex<=Nsexes;sex++)
        {
          //exploitation
          for (int age=1;age<=Nages-1;age++)
            {
              afterutotal=0.0;	
              for (int method=1;method<=Nmethods;method++)
                 {
                   afterutotal+=u(method,EndYear)*va(method,EndYear,sex,age)*utotal/utotalbefore;
                 }
              N(EndYear+1,sex,age+1)=N(EndYear,sex,age)*(1.0-afterutotal);
            }
            //Max Age
            afterutotal=0;	
            for (int method=1;method<=Nmethods;method++)
               {
                 afterutotal+=u(method,EndYear)*va(method,EndYear,sex,Nages)*utotal/utotalbefore;               
               }
            N(EndYear+1,sex,Nages)+=N(EndYear,sex,Nages)*(1.0-afterutotal);

            //natural mortality
            N(EndYear+1,sex)*=surv(sex);
              //recruitment = No residuals
            N(EndYear+1,sex,1)=(S(EndYear)/(alpha+beta*S(EndYear)))*RecFraction(sex);
          
	         //Vulnerable biomass by method
               for (method=1;method<=Nmethods;method++)
                 {
                   B(method,EndYear+1)+=(elem_prod(N(EndYear+1,sex),w(EndYear+1,sex))*va(method,EndYear+1,sex))*surv05(sex);
                 }
          }//sex
        //Spawning biomass
        S(EndYear+1)=N(EndYear+1,1)*fec(EndYear+1);
     
     

//=============================================================================================================================================

FUNCTION evaluate_priors

  Priorlike.initialize();
// Check that all the parameters have priors

// Virgin Recruitment
  if (active(R0)) Priorlike+=prior(R0,R0_prior); 

// Steepness parameter
  if (active(h)) Priorlike+=prior(h,h_prior); 

// Initial Age Structure - Residuals
  if (active(log_InitialDev))
    {
      if(p_log_InitialDev(4)==1) Priorlike+=norm2(log_InitialDev)/(2*square(p_log_InitialDev(6)));
    }

// Recruitment Residuals
  if (active(log_RecDev))
    {
      if (p_log_RecDev(4)==1) Priorlike+=0.5*norm2(log_RecDev)/(square(p_log_RecDev(6)));
    }

// Scaler of the Virgin Recruitment
  if (active(Rinit)) Priorlike+=prior(Rinit,Rinit_prior); 

// Natural mortality / Initial exploitation rate / plusscale parameter
  for (int sex=1;sex<=Nsexes;sex++)
    {
     if (active(M(sex))) Priorlike+=prior(M(sex),M_prior(sex)); 
     if (active(uinit(sex))) Priorlike+=prior(uinit(sex),uinit_prior(sex)); 
     if (active(plusscale(sex))) Priorlike+=prior(plusscale(sex),p_plusscale(sex));
    }
// FLEET: Selectivity Parameters
  for (int method=1;method<=Nmethods;method++)
    {
      if (active(Sfullest(method))) Priorlike+=prior(Sfullest(method),p_Sfullest(method)); 
      if (active(Sfulldelta(method))) Priorlike+=prior(Sfulldelta(method),p_Sfulldelta(method)); 
      if (active(log_varLest(method))) Priorlike+=prior(log_varLest(method),log_varLest_prior(method)); 
      if (active(log_varRest(method))) Priorlike+=prior(log_varRest(method),log_varRest_prior(method)); 
      if (active(errSfull(method))) Priorlike+=0.5*norm2(errSfull(method))/square(errSfull_prior(method,6)); 
      if (active(errvarL(method))) Priorlike+=0.5*norm2(errvarL(method))/square(errvarL_prior(method,6)); 
      if (active(errvarR(method))) Priorlike+=0.5*norm2(errvarR(method))/square(errvarR_prior(method,6)); 
    }

// FLEET: CPUE Parameters (Catchability & residuals for the catchability)
  for (int index=1;index<=NCPUEindex;index++)
    {
     if (active(log_qCPUE(index))) Priorlike+=prior(log_qCPUE(index),log_qCPUE_prior(index)); 
     if (active(qCPUEerr(index))) Priorlike+=0.5*norm2(qCPUEerr(index))/square(qCPUEerr_prior(index,6)); 
    }
  
// Survey: catchability & Selectivity Parameters
  for (index=1;index<=Nqsurveys;index++)
    {
     if (active(log_qsurvey(index))) Priorlike+=prior(log_qsurvey(index),log_qsurvey_prior(index)); 
     if (active(surveySfullest(index))) Priorlike+=prior(surveySfullest(index),surveySfull_prior(index)); 
     if (active(surveySfulldeltaest(index))) Priorlike+=prior(surveySfulldeltaest(index),p_surveySfulldelta(index)); 
     if (active(log_surveyvarL(index))) Priorlike+=prior(log_surveyvarL(index),log_surveyvarL_prior(index)); 
     if (active(log_surveyvarR(index))) Priorlike+=prior(log_surveyvarR(index),log_surveyvarR_prior(index)); 
    }
//=============================================================================================================================================

FUNCTION startup_screen
  cout<<"            *****************************************************\n";
  cout<<"            *                                                   *\n";
  cout<<"            *      Generalised Bayesian Age-Structured          *\n";
  cout<<"            *    Statistical Catch-at-Age and Length Model      *\n";
  cout<<"            *                                                   *\n";
  cout<<"            *  Designed by R. Hilborn, A. Parma, M. Maunder     *\n";
  cout<<"            *                 and b. eRnSt                      *\n";
  cout<<"            *                                                   *\n";
  cout<<"            * Programmed by M. Maunder, B. Ernst and A. Parma   *\n";                           
  cout<<"            *   Using AD Model Builder (C) Otter Research Ltd.  *\n";
  cout<<"            *                                                   *\n";
  cout<<"            *****************************************************\n";
  cout<<"\n";
  cout<<"\n";
  cout<<endl;
  cout<<"Running Model..........";

//=============================================================================================================================================
FUNCTION Proj_likelihoods

  dvariable sal1; 
//  if(CPUE_LikeType(1) != -1) sal1 = sum(elem_prod(nCPUELL,zeroCPUEtmp));

  if(sum(zeroCPUEtmp) > 0) sal1 = sum(elem_prod(CPUELL,zeroCPUEtmp));
    else sal1 = sum(CPUELL); 

  dvariable sal2;   
  if(sum(zeroCAtmp) > 0) sal2 = sum(elem_prod(nCALL,zeroCAtmp));    
    else sal2 = sum(nCALL);    

  dvariable sal3; 
  if(sum(zeroCLtmp) > 0) sal3 = sum(elem_prod(nCLLL , zeroCLtmp));
    else sal3 = sum(nCLLL);

  dvariable sal4; 
  if(sum(zeroSIndextmp) > 0) sal4 = sum(elem_prod(SurveyLL,zeroSIndextmp));  
    else sal4 = sum(SurveyLL);
         
  dvariable sal5; 
  if(sum(zeroSCAtmp) > 0) sal5 = sum(elem_prod(nsurveyCALL,zeroSCAtmp));
    else sal5 = sum(nsurveyCALL);
    
  dvariable sal6; 
  if(sum(zeroSCLtmp) > 0) sal6 = sum(elem_prod(nsurveyCLLL , zeroSCLtmp));
     else sal6 = sum(nsurveyCLLL);
  
  dvariable sal7; 
  if(sum(zeroSCLtmpns) > 0) sal7 = sum(elem_prod(surveynosexCLLL,zeroSCLtmpns));
    else sal7 = sum(surveynosexCLLL);  
  
  proj1<< sal1 <<" "<< sal2 <<" "<<sal3  <<" "<<sal4  <<" "<<sal5  <<" "<<sal6 <<" "<<sal7 <<" "<<negpen<<" "<<Priorlike<<" "<<f<<endl;
//=============================================================================================================================================

FUNCTION Proj_parameters
 // ***  Parameters  **
  int sex;
  int method;
   
  proj2<<R0<<" "<<h<<" "<<Rinit<<" ";
  for (sex=1;sex<=Nsexes;sex++)
    {  
      proj2<<M(sex)<<" ";
    }
  for (sex=1;sex<=Nsexes;sex++)
    {  
      proj2<<uinit(sex)<<" ";
    }
  for (sex=1;sex<=Nsexes;sex++)
    {  
      proj2<<plusscale(sex)<<" ";
    }
  for (method=1;method<=Nmethods; method++)
    {  
      proj2<<Sfullest(method)<<" ";
    }
  for (method=1;method<=Nmethods; method++)
    {  
      proj2<<Sfulldelta(method)<<" ";
    }
  for (method=1;method<=Nmethods; method++)
    {  
      proj2<<log_varLest(method)<<" ";
    }
  for (method=1;method<=Nmethods; method++)
    {  
      proj2<<log_varRest(method)<<" ";
    }
  for (method=1;method<=NCPUEindex; method++)
    {  
      proj2<<log_qCPUE(method)<<" ";
    }
  for (method=1;method<=Nqsurveys; method++)
    {  
      proj2<<log_qsurvey(method)<<" ";
    }
  for (method=1;method<=Nqsurveys; method++)
    {  
      proj2<<surveySfull(method)<<" ";
    }
  for (method=1;method<=Nqsurveys; method++)
    {  
      proj2<<surveySfulldeltaest(method)<<" ";
    }
  for (method=1;method<=Nqsurveys; method++)
    {  
      proj2<<log_surveyvarL(method)<<" ";
    }
  for (method=1;method<=Nqsurveys; method++)
    {  
      proj2<<log_surveyvarR(method)<<" ";
    }
    proj2<<endl;
//=============================================================================================================================================

FUNCTION Proj_residuals
// Residuals of the model
// Initial Age Structure Deviations / Recruitment Residuals / errSfull / errvarL / errvarR / qCPUEerr
  int method;
  proj3<<log_InitialDev<<" "<<log_RecDev<<" ";
  for (method=1;method<=Nmethods; method++)
    {  
      proj3<<errSfull(method)<<" ";
    } 
  for (method=1;method<=Nmethods; method++)
    {  
      proj3<<errvarL(method)<<" ";
    } 
  for (method=1;method<=Nmethods; method++)
    {  
      proj3<<errvarR(method)<<" ";
    } 
  for (method=1;method<=NCPUEindex; method++)
    {  
      proj3<<qCPUEerr(method)<<" ";
    }
  proj3<<endl;



FUNCTION Proj_spawnBiom  // Spawning Biomass

     proj4<<S<<endl;
    
  
FUNCTION Proj_expRate   // Exploitation rate
  int method;
  for (method=1;method<=Nmethods; method++)
     {  
       proj5<<u(method)<<" ";
     }
     proj5<<endl;


FUNCTION Proj_vulnBiom  // Vulnerable Biomass 
  int method;
  for (method=1;method<=Nmethods; method++)
     {  
       proj6<<B(method)<<" ";
     }
     proj6<<endl;


FUNCTION Proj_Recruits  // Recruits
  dvar_vector tmp(StartYear,EndYear);
  for (int sex=1; sex<=Nsexes;sex++)
     {
       for(int ix=StartYear; ix<=EndYear;ix++) tmp(ix)= tmp(ix) + N(ix,sex,1);
     } 
     
     proj7<<tmp<<endl;


FUNCTION Proj_Ro_So_VVB        // Lable for the virgin Recruitment / Virgin Spawning stock / Virgin vulnerable Biomass
	
	proj8<<R0<<" "<<S0<<" "<<VVB<<endl;

//=============================================================================================================================================

FUNCTION Projections
  dvar_vector tmp(StartYear,EndYear);
  for (int sex=1; sex<=Nsexes;sex++)
     {
       for(int ix=StartYear; ix<=EndYear;ix++) tmp(ix)= tmp(ix) + N(ix,sex,1);
     } 
// *************************************************
// Print LIKELIHOODS and POSTERIOR before projection
// *************************************************
  Proj_likelihoods();  // proj1 <- Print all the likelihoods to the file "mcmclike.out"
  Proj_parameters();   // proj2 <- Print out all parameters, but not residuals
  Proj_residuals();    // proj3 <- Print out all residuals
  Proj_spawnBiom();    // proj4 <- spawning Biomass
  Proj_expRate();      // proj5 <- exploitation rate
  Proj_vulnBiom();     // proj6 <- vulnerable Biomass 
  Proj_Recruits();     // proj7 <- recruits
  Proj_Ro_So_VVB();    // proj8 <- Ro / So / VVB 
   
// *********************************
// STARTS with the actual projection
// *********************************
  int method; 
  char mark;
  double randn(long int& n);
  dvector rec_dev(EndYear+1,ProjectYear+1);
  dvector tempdev(EndYear+1,ProjectYear+1);
  tempdev.fill_randn_ni(rec_seed);
  rec_dev =annualsigma*tempdev;
  rec_dev= exp(rec_dev);
  rec_dev*=biasRannual;
  dvariable rec;
  double error;
  double hr;
  double strategy;
  dvector assessment_dev(EndYear+1,ProjectYear+1);
  assessment_dev.fill_randn_ni(assessment_seed);
  assessment_dev *= assessment_cv*sqrt(1.-square(assessment_rho));
  double first_error;
  first_error = assessment_cv*randn(assessment_seed); 
  //find selectivity
  strategyva.initialize();
  dvariable sumu=0;
  // selectivity at age for the projections

  for (method=1;method<=Nmethods;method++)
     {
       for (sex=1;sex<=Nsexes;sex++)
          {
            for (int age=1;age<=Nages;age++)
               {
                 if (proj_gears==0)
                   {
                      strategyva(sex,age)+=strategyuproportion(method)*va(method,EndYear,sex,age);
                   }
                 else
                   {       
                      strategyva(sex,age)+=u(method,EndYear)*va(method,EndYear,sex,age);      
                   }
               }
          }
       sumu+=u(method,EndYear);
     }
     
  if (proj_gears!=0) strategyva=strategyva/sumu;
  
  // Weight and Fecundity for the projection --> EndYear +1 
  wproject=w(EndYear+1);
  fecproject=fec(EndYear+1);

  Sproject.initialize();
  Cproject.initialize();

  int Nstrategy= 1 + (EndStrategy-StartStrategy)/StepStrategy;
  if(StrategyType==2) Nstrategy=Nstrategy+1;
  proj9<<VVB<<" "<<B(1,StartYear)<<" ";
  proj10<<S0<<" "<<S(StartYear)<<" ";

  for (int istrategy=1;istrategy<=Nstrategy;istrategy++)
     {
     

        strategy = double(StartStrategy) + (double(istrategy)-1.0)*double(StepStrategy);        
        Nproject.initialize();
        Bproject.initialize();
        Nproject(EndYear+1)=N(EndYear+1);
        Sproject(istrategy,EndYear+1)=S(EndYear+1);

        for (int sex=1;sex<=Nsexes;sex++)
           {
              Nproject(EndYear+1,sex,1)*=rec_dev(EndYear+1);
              Bproject(EndYear+1)+=(elem_prod(Nproject(EndYear+1,sex),wproject(sex))*strategyva(sex))*surv05(sex); 
           }
        error = first_error;
        for (int year=EndYear+1; year<=ProjectYear;year++)
           {
             rec =(Sproject(istrategy,year)/(alpha+beta*Sproject(istrategy,year)))*rec_dev(year+1);
             for (int sex=1;sex<=Nsexes;sex++)
                {
                   //recruitment
                   Nproject(year+1,sex,1)=rec*RecFraction(sex);
                   Bproject(year+1)+=Nproject(year+1,sex,1)*wproject(sex,1)*strategyva(sex,1)*surv05(sex); 
                   Sproject(istrategy,year+1)+=Nproject(year+1,1,1)*fecproject(1);
                   
		   switch(StrategyType)
                     {
                       case 1:  
                          hr =bound_at_one(strategy/value(Bproject(year)));
                          Cproject(istrategy,year)=hr*Bproject(year);
                          break;
                       case 2:
                          error = assessment_rho*error + assessment_dev(year); 
                          hr =bound_at_one(strategy*exp(error));
                          Cproject(istrategy,year)= hr*Bproject(year);
                          break;
                       default:
                          break;
                     }
   
                   for (int age=1;age<=Nages-2;age++)
                      {
                         Nproject(year+1,sex,age+1)=Nproject(year,sex,age)*surv(sex)*(1-hr*strategyva(sex,age));
                         Bproject(year+1)+=Nproject(year+1,sex,age+1)*wproject(sex,age+1)*strategyva(sex,age+1)*surv05(sex);
	                 if (sex==1) Sproject(istrategy,year+1)+=Nproject(year+1,1,age+1)*fecproject(age+1);
                      }
                   //plus group

                   Nproject(year+1,sex,Nages)=(Nproject(year,sex,Nages-1)*(1-hr*strategyva(sex,Nages-1))+Nproject(year,sex,Nages)*(1-hr*strategyva(sex,Nages)))*surv(sex);
                   Bproject(year+1)+=Nproject(year+1,sex,Nages)*wproject(sex,Nages)*strategyva(sex,Nages)*surv05(sex);
                   if (sex==1) Sproject(istrategy,year+1)+=Nproject(year+1,1,Nages)*fecproject(Nages);
                 } //sex
             } //year

             // ************************
             // post Projection Printout
             // ************************
 
             switch(StrategyType)
                {
                  case 1:  
                     proj9<<Bproject<<" ";
 		     proj10<<Sproject(istrategy)<<" ";
                     for (year=EndYear+1; year<=ProjectYear;year++)
                        { 
		          proj13<<strategy<<" ";
		        
			} 
		          proj11<<Cproject(istrategy)<<" ";
			  
			  proj12<<tempdev<<endl;
		     
		     break;
                  case 2:
                     proj9<<Bproject<<" ";
 		     proj10<<Sproject(istrategy)<<" ";
                     for (year=EndYear+1; year<=ProjectYear;year++)
                        { 
		          proj13<<strategy<<" ";
		        
			} 
		          proj11<<Cproject(istrategy)<<" ";
			  
			  proj12<<tempdev<<endl;
		                          
                     break;
                  default:
                     break;
                }
       } // strategy
         proj9<<endl;
         proj10<<endl;
         proj11<<endl;
         proj12<<endl;
         proj13<<endl;

//=============================================================================================================================================
// *******************************
// LIKELIHOOD SECTION
// n* --> new likelihood functions
// *******************************
FUNCTION nSurveyLike
  SurveyLL.initialize();
  predsurvey.initialize();
  int pos;
  int year;
  int obs;
      for (int obs2=1; obs2<=nUsedSIndex;obs2++) // For the moment it loops over all the indeces that are turned on
          {
            pos=0; 
            obs=whatSIndex(obs2);  
	    dvar_vector predsurveytmp(1,nObsSIndex(obs)); 
            predsurvey=0.0;  
 	    predsurveytmp=0.0;
	      
            switch (survey_IndexType(obs))
              {
 		case 1: //Index in weight
                   if (survey_vType(obs)==1)  // age-dependent sel
                     {
                        for (int i=1; i<=nObsSIndex(obs);i++)
                          {
  
	                    pos++;
			    year = yearSIndex(obs,i);
                            for (int sex=1;sex<=Nsexes;sex++)
                               {
                                  predsurvey(pos)+=(elem_prod(N(year,sex),w(year,sex))*surveyva(obs,sex))*surv05(sex);
      		               }
		       
                            predsurvey(pos)*=qsurvey(obs);
 		            predsurveytmp(i)=predsurvey(pos);
	
	                  }

		      }
		      	  
		    else if (survey_vType(obs)==2)  //size-dep sel
                      {
                        for (int i=1; i<=nObsSIndex(obs);i++)
                           {
			     pos++;
			     year = yearSIndex(obs,i);
                             for (int sex=1;sex<=Nsexes;sex++)
                                {
                                  predsurvey(pos)+=(elem_prod(N(year,sex),survey_w(obs,sex))*surveyva(obs,sex))*surv05(sex);
                                }
			     predsurvey(pos)*=qsurvey(obs);
 			     predsurveytmp(i)=predsurvey(pos);
                           }
		       }   
       		 break;
        	 case 2: //Index in numbers
                 for (int i=1; i<=nObsSIndex(obs);i++)
                    {
		      pos++;
		      year = yearSIndex(obs,i);
                      for (int sex=1;sex<=Nsexes;sex++)
                         {
		            predsurvey(pos)+=(N(year,sex)*surveyva(obs,sex))*surv05(sex);
			 }
		      predsurvey(pos)*=qsurvey(obs);	 
  		      predsurveytmp(i)=predsurvey(pos);

     		    }
		 break;
         
	       }	 
	     SurveyLL(obs)= nlikelihood(ObsSIndex(obs), predsurveytmp, ObsCvSIndex(obs), survey_LikeType(obs)); //this cannot go at once because likelihood type can be different
        }


FUNCTION printSurveyLike //compute likelihood and predictions for Survey Indices with negative flag
  int pos;
  int year;
  int obs;

      for (obs=1; obs<=Nsurveyindex;obs++) // For the moment it loops over all the indeces that are turned on
        {
         pos=0;  
         if(survey_LikeType(obs) < 0)	  
	   {    
	     dvar_vector predsurveytmp(1,nObsSIndex(obs)); 
             predsurvey=0.0;  
  	     predsurveytmp=0.0;
	      
             switch (survey_IndexType(obs))
               {
 	 	case 1: //Index in weight
                   if (survey_vType(obs)==1)  // age-dependent sel
                     {
                        for (int i=1; i<=nObsSIndex(obs);i++)
                          {

			    pos++;
			    year = yearSIndex(obs,i);
                            for (int sex=1;sex<=Nsexes;sex++)
                               {
                                  predsurvey(pos)+=(elem_prod(N(year,sex),w(year,sex))*surveyva(obs,sex))*surv05(sex);
      		               }
                            predsurvey(pos)*=qsurvey(obs);
			    predsurveytmp(i)=predsurvey(pos);
	                  
	                  }
		      }
		      	  
		    else if (survey_vType(obs)==2)  //size-dep sel
                      {
                        for (int i=1; i<=nObsSIndex(obs);i++)
                           {
			     pos++;
			     year = yearSIndex(obs,i);
                             for (int sex=1;sex<=Nsexes;sex++)
                                {
                                  predsurvey(pos)+=(elem_prod(N(year,sex),survey_w(obs,sex))*surveyva(obs,sex))*surv05(sex);
                                }
			     predsurvey(pos)*=qsurvey(obs);
 			     predsurveytmp(i)=predsurvey(pos);
                           }
		       }   
       		 break;
        	 case 2: //Index in numbers
                 for (int i=1; i<=nObsSIndex(obs);i++)
                    {
		      pos++;
		      year = yearSIndex(obs,i);
                      for (int sex=1;sex<=Nsexes;sex++)
                         {
		            predsurvey(pos)+=(N(year,sex)*surveyva(obs,sex))*surv05(sex);
			 }
		      predsurvey(pos)*=qsurvey(obs);	 
  		      predsurveytmp(i)=predsurvey(pos);

     		    }
		 break;
	         
	       }	 
             // this code evaluate the likelihood by survey type. Can't do all at once, because likelihood type could be different  
  	     
	  SurveyLL(obs)= nlikelihood(ObsSIndex(obs), predsurveytmp, ObsCvSIndex(obs), -1.0*survey_LikeType(obs));

         }
	}
		 
FUNCTION nCPUELike  // this function computes the likelihood for CPUE with flag > 0
  npredCPUE.initialize();
  CPUELL.initialize();
  int pos;
  int year;
  int obs;
  int obsSelec;

      for (int obs2=1; obs2<=nUsedCPUE;obs2++) // different Surveys
         {
            pos=0; 
            
            obs=whatCPUE(obs2);  
	    obsSelec=whatSelecforCPUE(obs);
	        dvar_vector predCPUEtmp(1,nObsCPUE(obs)); 
		
 	        predCPUEtmp=0.0;
                npredCPUE=0.0;
	    for (int i=1; i<=nObsCPUE(obs);i++)
                {
                   pos++;
                   year = yearCPUE(obs,i);
      		       for (int sex=1;sex<=Nsexes;sex++)
                      {
                        npredCPUE(pos)+=(elem_prod(N(year,sex),w(year,sex))*va(obsSelec,year,sex))*surv05(sex);
      		          }
		           npredCPUE(pos)*=qCPUE(obs,year);  
                   predCPUEtmp(i)=npredCPUE(pos);
                }
             CPUELL(obs) = nlikelihood(ObsCPUE(obs), predCPUEtmp, ObsCvCPUE(obs), CPUE_LikeType(obs));
          
          }
//-----------------------------------------------------------------------------
FUNCTION printCPUELike  // this function allows to compute likelihoods for CPUEs that have negative flags
  npredCPUE.initialize();
  int pos;
  int year;
  int obsSelec;

      for (int obs=1; obs<=NCPUEindex;obs++) // different Surveys
        {
          pos=0;   
          if(CPUE_LikeType(obs) < 0)	  
 	     {    
               
		obsSelec=whatSelecforCPUE(obs);
		dvar_vector predCPUEtmp(1,nObsCPUE(obs)); 
 	        predCPUEtmp=0.0;
                npredCPUE=0.0;
	      
		for (int i=1; i<=nObsCPUE(obs);i++)
		    {
		       pos++;
		       year = yearCPUE(obs,i);
		       for (int sex=1;sex<=Nsexes;sex++)
			  {
			    npredCPUE(pos)+=(elem_prod(N(year,sex),w(year,sex))*va(obsSelec,year,sex))*surv05(sex);
			  }
			npredCPUE(pos)*=qCPUE(obs,year);  
			predCPUEtmp(i)=npredCPUE(pos);
		    }
	        CPUELL(obs) = nlikelihood(ObsCPUE(obs), predCPUEtmp, ObsCvCPUE(obs), -1.0*CPUE_LikeType(obs));

 	      }  	 
            }
 
 //-----------------------------------------------------------------------------
 // howManySCA 		<-- how many different indices are there in the CA data.
 // whichSCA 	     	<-- which indices are there in the data
 // whichLikeSCA     	<-- which indices are there in the data and Used (likelihood flag)
 // nUsedSCA     	<-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroSCAtmp   	<-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearSCA       	<-- DATA: years for which catch at age is available per index
 // ObsSsizeSCA   	<-- DATA: sample size
 // ObsSCA        	<-- DATA: proportions at age

FUNCTION nsurveyCALike
  nsurveyCALL.initialize();
  nsurveyCAp.initialize();
   int year;
   int obs;
   for (int obs2=1;obs2<=nUsedSCA;obs2++)     //howManySCA;obs++)
      {
        obs = whichLikeSCA(obs2);
	for (int i=1; i<=nObsSCA(obs);i++)
           {
             year = yearSCA(obs,i);
             dvar_matrix tempNsexNA(1,Nsexes,1,Nages);
             dvar_matrix tempNsexCA(1,Nsexes,1,PoolAge);
            
	     for (int sex=1; sex<=Nsexes;sex++)
                {    
                  tempNsexNA(sex)=elem_prod(surveyva(whichSCA(obs),sex),N(year,sex))*surv05(sex);
                }

                tempNsexNA = tempNsexNA / sum(tempNsexNA);

             for (sex=1; sex<=Nsexes;sex++)
                {
                  
		  tempNsexCA(sex)=tempNsexNA(sex) * AerrorApool;  //correct for errors in age misclassification
                  
		  for (int age=1;age<=PoolAge;age++)
                     {
                        nsurveyCAp(obs,i,(sex-1)*PoolAge + age) = tempNsexCA(sex,age);
                     }
                }
            }
         nagestemp=PoolAge*Nsexes;
         switch (surveyCA_LikeType(whichSCA(obs)))
            {
	       case 11:
                 nsurveyCALL(obs) = multinomial(ObsSCA(obs) ,nsurveyCAp(obs) ,nagestemp , nObsSCA(obs), ObsSsizeSCA(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 nsurveyCALL(obs) = nrobust_p(ObsSCA(obs) ,nsurveyCAp(obs) ,nagestemp , nObsSCA(obs), ObsSsizeSCA(obs)); //,ObsSsizeSCA(obs)      
                 break;
	     }
       }//obs

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
FUNCTION printnsurveyCALike
  surveyCALL.initialize();
   int year;
   int obs;
   for (int obs2=1;obs2<=howManySCA;obs2++)     
      {
       obs = whichSCA(obs2);

       if(surveyCA_LikeType(obs) != 0)
         {
	   for (int i=1; i<=nObsSCA(obs);i++)
             {
              year = yearSCA(obs,i);
              dvar_matrix tempNsexNA(1,Nsexes,1,Nages);
              dvar_matrix tempNsexCA(1,Nsexes,1,PoolAge);
 
            
	      for (int sex=1; sex<=Nsexes;sex++)
                 {    
                   tempNsexNA(sex)=elem_prod(surveyva(obs,sex),N(year,sex))*surv05(sex);
                 }

                 tempNsexNA = tempNsexNA / sum(tempNsexNA);

              for (sex=1; sex<=Nsexes;sex++)
                 {
               	   tempNsexCA(sex)=tempNsexNA(sex) * AerrorApool;  //correct for errors in age misclassification
                  
		   for (int age=1;age<=PoolAge;age++)
                      {
                         nsurveyCAp(obs,i,(sex-1)*PoolAge + age) = tempNsexCA(sex,age);
			 surveyCApredtemp(obs,age)=nsurveyCAp(obs,i,(sex-1)*PoolAge + age);
                      }
                 }
             }
         nagestemp=PoolAge*Nsexes;


	  int liketype;
          liketype=surveyCA_LikeType(obs);
          if(surveyCA_LikeType(obs)<0) liketype*=-1;   	  
          switch (liketype)
              {
	       case 11:
                 surveyCALL(obs) = multinomial(ObsSCA(obs) ,nsurveyCAp(obs) ,nagestemp , nObsSCA(obs), ObsSsizeSCA(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 surveyCALL(obs) = nrobust_p(ObsSCA(obs) ,nsurveyCAp(obs) ,nagestemp , nObsSCA(obs), ObsSsizeSCA(obs)); //,ObsSsizeSCA(obs)      
                break;

	      }
	  }    
       }//obs

    surveyCALike();
//-----------------------------------------------------------------------------
FUNCTION surveyCALike
  surveyCAp.initialize();
  for (int obs=1;obs<=NsurveyCA;obs++)
  {
   int nsurvey=surveyCA(obs,1);
   int year=surveyCA(obs,2);
   dvar_matrix tempNsexNA(1,Nsexes,1,Nages);

      for (int sex=1; sex<=Nsexes;sex++)
      {    
        tempNsexNA(sex)=elem_prod(surveyva(nsurvey,sex),N(year,sex))*surv05(sex);
      }
      tempNsexNA=tempNsexNA/sum(tempNsexNA);
      for (sex=1; sex<=Nsexes;sex++)
      {
         surveyCAp(obs,sex)=tempNsexNA(sex)*AerrorApool;
         for (int age=1;age<=PoolAge;age++)
         {
         surveyCApredtemp(obs,(sex-1)*PoolAge+age)=surveyCAp(obs,sex,age);
         }
      }
      nagestemp=PoolAge*Nsexes;
      tautemp=surveyCA(obs,3);
  }//obs
 //-----------------------------------------------------------------------------
 // FLEET CATCH AT AGE CALCULATION
 //-----------------------------------------------------------------------------
 // howManyCA 		<-- how many different indices are there in the CA data.
 // whichCA 	     	<-- which indices are there in the data
 // whichLikeCA     	<-- which indices are there in the data
 // nUsedCA     	<-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroCAtmp   	<-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearCA       	<-- DATA: years for which catch at age is available per index
 // ObsSsizeCA   	<-- DATA: sample size
 // ObsCA        	<-- DATA: proportions at age
FUNCTION nCALike
  nCALL.initialize();
  nCAp.initialize();
   int year;
   int obs;
   for (int obs2=1;obs2<=nUsedCA;obs2++)// this is related to the gears
      {
        obs=whichLikeCA(obs2);
        for (int i=1; i<=nObsCA(obs);i++)
           {
             year = yearCA(obs,i);
             dvar_matrix tempNsexNA(1,Nsexes,1,Nages);
             dvar_matrix tempNsexCA(1,Nsexes,1,PoolAge);
            
	     for (int sex=1; sex<=Nsexes;sex++)
                {    
                  tempNsexNA(sex)=elem_prod(va(whichCA(obs),year,sex),N(year,sex))*surv05(sex);
                }
                tempNsexNA = tempNsexNA / sum(tempNsexNA);
             for (sex=1; sex<=Nsexes;sex++)
                {
                  tempNsexCA(sex)=tempNsexNA(sex) * AerrorApool;  //correct for errors in age misclassification
                  for (int age=1;age<=PoolAge;age++)
                     {
                        nCAp(obs,i,(sex-1)*PoolAge + age) = tempNsexCA(sex,age);
                     }
                }
            }
           nagestemp=PoolAge*Nsexes;


         switch (CA_LikeType(obs))
            {
	       case 11:
                 nCALL(obs) = multinomial(ObsCA(obs) ,nCAp(obs) ,nagestemp , nObsCA(obs), ObsSsizeCA(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 nCALL(obs) = nrobust_p(ObsCA(obs) ,nCAp(obs) ,nagestemp , nObsCA(obs), ObsSsizeCA(obs)); //,ObsSsizeSCA(obs)      
		 break;
	     }

       }//obs
//-----------------------------------------------------------------------------
FUNCTION printnCALike
   CALL.initialize();
   int year;
   int obs;
   for (int obs2=1;obs2<=howManyCA;obs2++)// this is related to the gears
      {
        obs=whichCA(obs2);        
        if(CA_LikeType(obs) != 0)
          {

           for (int i=1; i<=nObsCA(obs);i++)
             {
               year = yearCA(obs,i);
               dvar_matrix tempNsexNA(1,Nsexes,1,Nages);
               dvar_matrix tempNsexCA(1,Nsexes,1,PoolAge);
            
	       for (int sex=1; sex<=Nsexes;sex++)
                 {    
                   tempNsexNA(sex)=elem_prod(va(obs,year,sex),N(year,sex))*surv05(sex);
                 }
               tempNsexNA = tempNsexNA / sum(tempNsexNA);
               for (sex=1; sex<=Nsexes;sex++)
                 {
                   tempNsexCA(sex)=tempNsexNA(sex) * AerrorApool;  //correct for errors in age misclassification
                   for (int age=1;age<=PoolAge;age++)
                       {
                         nCAp(obs,i,(sex-1)*PoolAge + age) = tempNsexCA(sex,age);
                      }
                 }
             }
            nagestemp=PoolAge*Nsexes;
	  int liketype;
          liketype=CA_LikeType(obs);
          if(CA_LikeType(obs)<0) liketype=-1*CA_LikeType(obs);   	  
          switch (liketype)
            {
	       case 11:
                 CALL(obs) = multinomial(ObsCA(obs) ,nCAp(obs) ,nagestemp , nObsCA(obs), ObsSsizeCA(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 CALL(obs) = nrobust_p(ObsCA(obs) ,nCAp(obs) ,nagestemp , nObsCA(obs), ObsSsizeCA(obs)); //,ObsSsizeSCA(obs)      
		 break;
	     }
	}     
       }//obs
    CALike();
//-----------------------------------------------------------------------------
FUNCTION CALike
  for (int obs=1;obs<=NCA;obs++)
    {
      int method=CA(obs,1);
      int year=CA(obs,2);
      dvar_matrix tempNsexNA(1,Nsexes,1,Nages);
          for (int sex=1; sex<=Nsexes;sex++)
            {
              tempNsexNA(sex)=elem_prod(va(method,year,sex),N(year,sex))*surv05(sex); 
            }
          tempNsexNA=tempNsexNA/sum(tempNsexNA);
          for (sex=1; sex<=Nsexes;sex++)
            {
              CAp(obs,sex)=tempNsexNA(sex)*AerrorApool;
            }
          for (sex=1; sex<=Nsexes;sex++)
            {
             for (int age=1;age<=PoolAge;age++)
               {
                 CApredtemp(obs,(sex-1)*PoolAge+age)=CAp(obs,sex,age);
               }
            }
          nagestemp=PoolAge*Nsexes;
          tautemp=CA(obs,3);
     }//obs
//-----------------------------------------------------------------------------
 // howManySCL    <-- how many different indices are there in the CA data.
 // whichSCL      <-- which indices are there in the data
 // whichLikeSCL  <-- LIKELIHOOD: which indices are used, following the indeces of whichSCL
 // nUsedSCL      <-- LIKELIHOOD: number of used indices (from the likelihood flag side)
 // zeroSCLtmp    <-- LIKELIHOOD: 0/1 depending on the likelihood flags
 // yearSCL       <-- DATA: years for which catch at age is available per index
 // ObsSsizeSCL   <-- DATA: sample size
 // ObsSCL        <-- DATA: proportions at age

FUNCTION nsurveyCLLike
  nsurveyCLLL.initialize();
  nsurveyCLp.initialize();
 
   int obs;
   int year;
   for (int obs2=1;obs2<=nUsedSCL;obs2++)     
      {
        obs = whichLikeSCL(obs2);
	for (int i=1; i<=nObsSCL(obs);i++)
           {
             year = yearSCL(obs,i);
 
             for (int sex=1; sex<=Nsexes;sex++)
                {
                  for (int age=1;age<=Nages;age++)
                     {
                       for (int length=1;length<iPoolLength;length++)
                          {
                            nsurveyCLp(obs,i,(sex-1)*iPoolLength+length)+=surveyva(whichSCL(obs),sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }
                       for (length=iPoolLength;length<=Nlens;length++)
                          {
                            nsurveyCLp(obs,i,(sex-1)*iPoolLength+iPoolLength)+=surveyva(whichSCL(obs),sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }   //length
                      }  //age   
                 }  //sex

              nsurveyCLp(obs,i)=nsurveyCLp(obs,i)/sum(nsurveyCLp(obs,i));
           } //nobs	 
          nagestemp=iPoolLength*Nsexes;
          switch (surveyCL_LikeType(whichSCL(obs)))
            {
	       case 11:
                 nsurveyCLLL(obs) = multinomial(ObsSCL(obs) ,nsurveyCLp(obs) ,nagestemp , nObsSCL(obs), ObsSsizeSCL(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 nsurveyCLLL(obs) = nrobust_p(ObsSCL(obs) ,nsurveyCLp(obs) ,nagestemp , nObsSCL(obs), ObsSsizeSCL(obs)); //,ObsSsizeSCA(obs)      
		 break;
	     }
      }//obs
      
FUNCTION printnsurveyCLLike
  surveyCLLL.initialize();
  nsurveyCLp.initialize();
   int obs;
   int year;


   for (int obs2=1;obs2<=howManySCL;obs2++)     
      {
        obs = whichSCL(obs2);

       if(surveyCL_LikeType(whichSCL(obs)) != 0)
         {
         for (int i=1; i<=nObsSCL(obs);i++)
           {
             year = yearSCL(obs,i);
 
             for (int sex=1; sex<=Nsexes;sex++)
                {
                  for (int age=1;age<=Nages;age++)
                     {
                       for (int length=1;length<iPoolLength;length++)
                          {
                            nsurveyCLp(obs,i,(sex-1)*iPoolLength+length)+=surveyva(whichSCL(obs),sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }
                       for (length=iPoolLength;length<=Nlens;length++)
                          {
                            nsurveyCLp(obs,i,(sex-1)*iPoolLength+iPoolLength)+=surveyva(whichSCL(obs),sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }   //length
                      }  //age   
                 }  //sex

              nsurveyCLp(obs,i)=nsurveyCLp(obs,i)/sum(nsurveyCLp(obs,i));
           } //nobs	 
          nagestemp=iPoolLength*Nsexes;

	  int liketype;
	  liketype=surveyCL_LikeType(whichSCL(obs));
          if(surveyCL_LikeType(whichSCL(obs))<0) liketype*=-1;   	  
          switch (liketype)
            {
	       case 11:
                 surveyCLLL(obs) = multinomial(ObsSCL(obs) ,nsurveyCLp(obs) ,nagestemp , nObsSCL(obs), ObsSsizeSCL(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 surveyCLLL(obs) = nrobust_p(ObsSCL(obs) ,nsurveyCLp(obs) ,nagestemp , nObsSCL(obs), ObsSsizeSCL(obs)); //,ObsSsizeSCA(obs)      
		 break;
	    }
	}     

      }//obs      
    surveyCLLike();  
     

FUNCTION surveyCLLike
  surveyCLp.initialize();
  for (int obs=1;obs<=NsurveyCL;obs++)
  {
   int survey=surveyCL(obs,1);
   int year=surveyCL(obs,2);
      for (int sex=1; sex<=Nsexes;sex++)
      {
        for (int age=1;age<=Nages;age++)
          {
            for (int length=1;length<iPoolLength;length++)
             {
               surveyCLp(obs,(sex-1)*iPoolLength+length)+=surveyva(survey,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
             }
            for (length=iPoolLength;length<=Nlens;length++)
             {
               surveyCLp(obs,(sex-1)*iPoolLength+iPoolLength)+=surveyva(survey,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
             }   //length
          }  //age   
      }  //sex
      surveyCLp(obs)=surveyCLp(obs)/sum(surveyCLp(obs));
      nagestemp=iPoolLength*Nsexes;
      tautemp=surveyCL(obs,3);
  }//obs

          
//-----------------------------------------------------------------------------

FUNCTION nsurveynosexCLLike
  nsurveynosexCLLL.initialize();
  nsurveynosexCLp.initialize();
   int obs;
   int year;
   for (int obs2=1;obs2<=nUsedSCLns;obs2++)     
      {
        obs = whichLikeSCLns(obs2);
	for (int i=1; i<=nObsSCLns(obs);i++)
           {
             year = yearSCLns(obs,i);
 
             for (int sex=1; sex<=Nsexes;sex++)
                {
                  for (int age=1;age<=Nages;age++)
                     {
                       for (int length=1;length<iPoolLength;length++)
                          {
                             nsurveynosexCLp(obs,i,length)+=surveyva(whichSCLns(obs),sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }
                       for (length=iPoolLength;length<=Nlens;length++)
                          {
                             nsurveynosexCLp(obs,i,iPoolLength)+=surveyva(whichSCLns(obs), sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }   //length
                      }  //age   
                 }  //sex
             nsurveynosexCLp(obs,i)=nsurveynosexCLp(obs,i)/sum(nsurveynosexCLp(obs,i));
           } //nobs	 

          nagestemp=iPoolLength*Nsexes;
          switch (surveynosexCL_LikeType(whichSCLns(obs)))
            {
	       case 11:
                 nsurveynosexCLLL(obs) = multinomial(ObsSCLns(obs) ,nsurveynosexCLp(obs) ,nagestemp , nObsSCLns(obs), ObsSsizeSCLns(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 nsurveynosexCLLL(obs) = nrobust_p(ObsSCLns(obs) ,nsurveynosexCLp(obs) ,nagestemp , nObsSCLns(obs), ObsSsizeSCLns(obs)); //,ObsSsizeSCA(obs)      
		 break;
	     }
      }//obs2

FUNCTION printnsurveynosexCLLike
  nsurveynosexCLLL.initialize();
  nsurveynosexCLp.initialize();
   int obs;
   int year;
   for (int obs2=1;obs2<=howManySCLns;obs2++)     
      {
        obs = whichSCLns(obs2);
        if(surveynosexCL_LikeType(whichSCLns(obs)) != 0)
          {

	    for (int i=1; i<=nObsSCLns(obs);i++)
	       {
		 year = yearSCLns(obs,i);
 
		 for (int sex=1; sex<=Nsexes;sex++)
		    {
		      for (int age=1;age<=Nages;age++)
			 {
			   for (int length=1;length<iPoolLength;length++)
			      {
				 nsurveynosexCLp(obs,i,length)+=surveyva(whichSCLns(obs),sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
			      }
			   for (length=iPoolLength;length<=Nlens;length++)
			      {
				 nsurveynosexCLp(obs,i,iPoolLength)+=surveyva(whichSCLns(obs), sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
			      }   //length
			  }  //age   
		     }  //sex
		 nsurveynosexCLp(obs,i)=nsurveynosexCLp(obs,i)/sum(nsurveynosexCLp(obs,i));
	       } //nobs	 

          nagestemp=iPoolLength*Nsexes;
	  int liketype;
	  liketype=surveynosexCL_LikeType(whichSCLns(obs));
          if(surveynosexCL_LikeType(whichSCLns(obs))<0) liketype*=-1;   	  
          switch (liketype)
            {
	       case 11:
                 surveynosexCLLL(obs) = multinomial(ObsSCLns(obs) ,nsurveynosexCLp(obs) ,nagestemp , nObsSCLns(obs), ObsSsizeSCLns(obs)); //,ObsSsizeSCA(obs)      
                 break;
               case 12:
                 surveynosexCLLL(obs) = nrobust_p(ObsSCLns(obs) ,nsurveynosexCLp(obs) ,nagestemp , nObsSCLns(obs), ObsSsizeSCLns(obs)); //,ObsSsizeSCA(obs)      
		 break;
	     }
         } //if
     }//obs2
     surveynosexCLLike();

//-----------------------------------------------------------------------------
FUNCTION surveynosexCLLike
  surveynosexCLp.initialize();
  for (int obs=1;obs<=NsurveynosexCL;obs++)
  {
   int survey=surveynosexCL(obs,1);
   int year=surveynosexCL(obs,2);
      for (int sex=1; sex<=Nsexes;sex++)
      {
      for (int age=1;age<=Nages;age++)
      {
         for (int length=1;length<iPoolLength;length++)
         {
            surveynosexCLp(obs,length)+=surveyvl(survey,sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
         }
         for (length=iPoolLength;length<=Nlens;length++)
         {
            surveynosexCLp(obs,iPoolLength)+=surveyvl(survey, sex,length)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
         }
      }
      }
      surveynosexCLp(obs)=surveynosexCLp(obs)/sum(surveynosexCLp(obs));
      tautemp=surveynosexCL(obs,3);
      nagestemp=iPoolLength;
  }//obs

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Likelihood for the Fishery
//-----------------------------------------------------------------------------
FUNCTION nCLLike
  nCLLL.initialize();
  nCLp.initialize();
  int obs;
  int year;
  for (int obs2=1;obs2<=nUsedCL;obs2++)
     {
       obs =  whichLikeCL(obs2);
       for (int i=1; i<=nObsCL(obs);i++)
          {
            year = yearCL(obs,i);
            for (int sex=1; sex<=Nsexes;sex++)
               {
                 for (int age=1;age<=Nages;age++)
                    {
                      for (int length=1;length<iPoolLength;length++)
                         {
                           nCLp(obs,i,(sex-1)*iPoolLength+length)+=va(whichCL(obs),year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                         }
                       for (length=iPoolLength;length<=Nlens;length++)
                          {
                            nCLp(obs,i,(sex-1)*iPoolLength+iPoolLength)+=va(whichCL(obs),year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }   //length
                      }  //age   
                 }  //sex
	     nCLp(obs,i)=nCLp(obs,i)/sum(nCLp(obs,i));
        
	     } //nobs	 
        nagestemp=iPoolLength*Nsexes;
                   
        switch (CL_LikeType(whichCL(obs)))
            {
	       case 11:
                 nCLLL(obs) = multinomial(ObsCL(obs) ,nCLp(obs) ,nagestemp , nObsCL(obs), ObsSsizeCL(obs));
                 break;
               case 12:
                 nCLLL(obs) = nrobust_p(ObsCL(obs) ,nCLp(obs) ,nagestemp , nObsCL(obs), ObsSsizeCL(obs));       
		 break;
	     }
      }//obs



FUNCTION printnCLLike
  CLLL.initialize();
  nCLp.initialize();
   int obs;
   int year;

   for (int obs2=1;obs2<=howManyCL;obs2++)     
      {
        obs = whichCL(obs2);
        if(CL_LikeType(whichSCL(obs)) != 0)
         {
         for (int i=1; i<=nObsCL(obs);i++)
           {
             year = yearCL(obs,i);
 
            for (int sex=1; sex<=Nsexes;sex++)
               {
                 for (int age=1;age<=Nages;age++)
                    {
                      for (int length=1;length<iPoolLength;length++)
                         {
                           nCLp(obs,i,(sex-1)*iPoolLength+length)+=va(whichCL(obs),year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                         }
                       for (length=iPoolLength;length<=Nlens;length++)
                          {
                           nCLp(obs,i,(sex-1)*iPoolLength+iPoolLength)+=va(whichCL(obs),year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                          }   //length
                      }  //age   
                 }  //sex
              nCLp(obs,i)=nCLp(obs,i)/sum(nCLp(obs,i));
           } //nobs	 

          nagestemp=iPoolLength*Nsexes;
	  int liketype;
	  liketype=CL_LikeType(whichCL(obs));
          if(CL_LikeType(whichCL(obs))<0) liketype*=-1;   	  
          switch (liketype)
            {
	       case 11:
                 CLLL(obs) = multinomial(ObsCL(obs) ,nCLp(obs) ,nagestemp , nObsCL(obs), ObsSsizeCL(obs));
                 break;
               case 12:
                 CLLL(obs) = nrobust_p(ObsCL(obs) ,nCLp(obs) ,nagestemp , nObsCL(obs), ObsSsizeCL(obs));  
		 break;
	    }
	}     
      }//obs      
    CLLike();  


         
//-----------------------------------------------------------------------------
FUNCTION CLLike
  CLp.initialize();
  for (int obs=1;obs<=NCL;obs++)
    {
      int method=CL(obs,1);
      int year=CL(obs,2);
      for (int sex=1; sex<=Nsexes;sex++)
         {
           for (int age=1;age<=Nages;age++)
              {
                //MM This should be vl not va same with survey
                for (int length=1;length<iPoolLength;length++)
                   {
                      CLp(obs,(sex-1)*iPoolLength+length)+=va(method,year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                   }
                for (length=iPoolLength;length<=Nlens;length++)
                   {
                      CLp(obs,(sex-1)*iPoolLength+iPoolLength)+=va(method,year,sex,age)*N(year,sex,age)*LgivenA(sex,age,length)*surv05(sex);
                   }
              }//age
          }//sex
      CLp(obs)=CLp(obs)/sum(CLp(obs));
      nagestemp=iPoolLength*Nsexes;
      tautemp=CL(obs,3);
   }//obs
//-----------------------------------------------------------------------------


FUNCTION printOldStuff
   //compute predictions and likelihoods for all indeces with flag equal -1 or positive
   printSurveyLike();   
   printCPUELike();
   printnsurveyCALike();
   printnCALike(); 
   printnsurveyCLLike();
   printnCLLike();
   printnsurveynosexCLLike();
   
//   if(prflagSCLtmp > 0) //Survey CL Likelihood
//   if(prflagSCLtmpns > 0) //Survey no sex CL Likelihood
//   if(prflagCAtmp > 0) //Fleet CA Likelihood
//   if(prflagCLtmp > 0) //Fleet CL Likelihood


FUNCTION Output_File
  ofstream out1("results.dat",ios::trunc);
  out1<<"**Dimensioning_Parameters**"<<endl<<endl;
  out1<<"StartYear "<<StartYear<<endl;
  out1<<"EndYear "<<EndYear<<endl;
  out1<<"Nsexes "<<Nsexes<<endl;
  out1<<"Nages "<<Nages<<endl;
  out1<<"Nmethods "<<Nmethods<<endl;
  out1<<"NCPUEindex "<<NCPUEindex<<endl;
  out1<<"Nsurveyindex "<<Nsurveyindex<<endl;
  out1<<"First_length "<<FirstL<<endl;
  out1<<"Length_class_increment "<<IncL<<endl;
  out1<<"Number_of_length_classes "<<Nlens<<endl<<endl<<endl;
  out1<<"**Likelihoods**"<<endl<<endl;
  out1<<"CPUE "<<CPUELL<<endl;
  out1<<"Survey_Index "<<SurveyLL<<endl;
  out1<<"C@A_Commercial "<<CALL<<endl;
  out1<<"C@A_survey "<<surveyCALL<<endl;
  out1<<"C@L_Commercial "<<CLLL<<endl;
  out1<<"C@L_no_sex_data_survey "<<surveynosexCLLL<<endl;
  out1<<"C@L_data_survey "<<surveyCLLL<<endl;
  out1<<"Prior_penalties "<<Priorlike<<endl<<endl<<endl;
  out1<<"**Parameters**"<<endl<<endl;
  out1<<"R0 "<<R0<<endl;
  out1<<"h "<<h<<endl<<endl;
  out1<<"Sex_specific"<<endl;
//OjO 
  out1<<"M ";  
  for (int sex=1;sex<=Nsexes;sex++)
    {  
      out1<<M(sex)<<" ";
    }
    out1<<endl;

  out1<<"Rinit "<<Rinit<<endl;

//OjO  
  out1<<"uinit "; 
  for (sex=1;sex<=Nsexes;sex++)
    {  
      out1<<uinit(sex)<<" ";
    }
    out1<<endl;

//OjO
  out1<<"plusscale ";
  for (sex=1;sex<=Nsexes;sex++)
    {  
      out1<<plusscale(sex)<<" ";
    }
    out1<<endl;

  out1<<"Method_specific"<<endl;

//OjO
  out1<<"Sfullest ";
  for (int method=1;method<=Nmethods; method++)
    {  
      out1<<Sfullest(method)<<" ";
    }
  out1<<endl;


//OjO
  out1<<"log_varLest ";
  for (method=1;method<=Nmethods; method++)
    {  
      out1<<log_varLest(method)<<" ";
    }
  out1<<endl;

//OjO
  out1<<"log_varRest ";  

  for (method=1;method<=Nmethods; method++)
    {  
      out1<<log_varRest(method)<<" ";
    }
  out1<<endl;
  out1<<endl;
  out1<<"Method_specific_and_annual"<<endl;
  out1<<"errSfull ";
  for (method=1;method<=Nmethods; method++)
    {  
      out1<<errSfull(method)<<endl;
    } 
  
  
  out1<<"errvarL ";
  for (method=1;method<=Nmethods; method++)
    {  
      out1<<errvarL(method)<<endl;
    } 
  
  out1<<"errvarR ";
  for (method=1;method<=Nmethods; method++)
    {  
      out1<<errvarR(method)<<endl;
    } 
   out1<<endl;
   
  out1<<"CPUE_index_specific"<<endl;

//OjO
  out1<<"log_qCPUE ";
  for (method=1;method<=NCPUEindex; method++)
    {  
      out1<<log_qCPUE(method)<<" ";
    }
  out1<<endl;
  out1<<endl;

//OjO
  out1<<"CPUE_index_specific_and_annual"<<endl;
  out1<<"qCPUEerr ";
  for (method=1;method<=NCPUEindex; method++)
    {  
      out1<<qCPUEerr(method)<<" ";
    }
  out1<<endl;
 
  out1<<"q "<<qCPUE<<endl<<endl;
  out1<<"Survey_index_specific"<<endl;


//OjO
  out1<<"log_qsurvey ";
  for (method=1;method<=Nqsurveys; method++)
    {  
      out1<<log_qsurvey(method)<<" ";
    }
  out1<<endl;

//OjO
  out1<<"surveySfull ";
  for (method=1;method<=Nqsurveys; method++)
    {  
      out1<<surveySfull(method)<<" ";
    }
  out1<<endl;

//OjO
  out1<<"log_surveyvarL ";
  for (method=1;method<=Nqsurveys; method++)
    {  
      out1<<log_surveyvarL(method)<<" ";
    }
  out1<<endl;

//OjO
  out1<<"log_surveyvarR ";
  for (method=1;method<=Nqsurveys; method++)
    {  
      out1<<log_surveyvarR(method)<<" ";
    }
  out1<<endl;
  out1<<endl;


  out1<<"Recruitment_residuals"<<endl;
  out1<<"log_InitialDev "<<log_InitialDev<<endl;
  out1<<"log_RecDev "<<log_RecDev<<endl<<endl<<endl;
  char mark;

  out1<<"**Priors**"<<endl;  
  out1<<"Priortype_0(or_non_defined_value)=uniform,1=normal,2=lognormal"<<endl;
  out1<<" Phase LB UB PriorType mean cv initial_value"<<endl<<endl;
  out1<<"R0_prior "<<R0_prior<<endl;
  out1<<"h_prior "<<h_prior<<endl<<endl;
  out1<<"Sex_specific"<<endl;
  out1<<"M_prior "<<M_prior<<endl;
  out1<<"Rinit_prior "<<Rinit_prior<<endl;
  out1<<"uinit_prior "<<uinit_prior<<endl;
  out1<<"p_plusscale "<<p_plusscale<<endl;
  out1<<"Method_specific"<<endl;
  out1<<"p_Sfullest "<<p_Sfullest<<endl;
  out1<<"p_Sfulldelta "<<p_Sfulldelta<<endl;
  out1<<"log_varLest_prior "<<log_varLest_prior<<endl;
  out1<<"log_varRest_prior "<<log_varRest_prior<<endl<<endl;  
  out1<<"Method_specific_and_annual"<<endl;
  out1<<"errSfull_prior "<<errSfull_prior<<endl;
  out1<<"errvarL_prior "<<errvarL_prior<<endl;
  out1<<"errvarR_prior "<<errvarR_prior<<endl<<endl;
  out1<<"CPUE_index_specific"<<endl;
  out1<<"log_qCPUE_prior "<<log_qCPUE_prior<<endl<<endl;
  out1<<"CPUE_index_specific_and_annual"<<endl;
  out1<<"qCPUEerr_prior "<<qCPUEerr_prior<<endl<<endl;
  out1<<"Survey_index_specific"<<endl;
  out1<<"log_qsurvey_prior "<<log_qsurvey_prior<<endl;
  out1<<"surveySfull_prior "<<surveySfull_prior<<endl;
  out1<<"p_surveySfulldelta "<<p_surveySfulldelta<<endl;
  out1<<"log_surveyvarL_prior "<<log_surveyvarL_prior<<endl;  
  out1<<"log_surveyvarR_prior "<<log_surveyvarR_prior<<endl<<endl;
  out1<<"Recruitment_residuals"<<endl;
  out1<<"p_log_InitialDev "<<p_log_InitialDev<<endl;
  out1<<"p_log_RecDev "<<p_log_RecDev<<endl<<endl<<endl;
  out1<<"**Age-structured_values**"<<endl<<endl;
  out1<<"Mean_Length_by_sex_and_age"<<endl;
  out1<<Lmean<<endl<<endl;
  out1<<"cv_of_Length_by_sex_and_age"<<endl;
  out1<<Lsd<<endl<<endl;
  out1<<"Weight_by_year,sex_and_age"<<endl;
  out1<<w<<endl<<endl;
  out1<<"Fecundity_by_year_and_age"<<endl;
  out1<<fec<<endl<<endl;
  out1<<"Commercial_age-specific_selectivity_by_method,Year,sex_and_age"<<endl;

//OjO
  for (method=1;method<=Nmethods;method++)
  {  
   out1<<va(method)<<endl;
  }
  out1<<endl;
  out1<<"Survey_age-specific_selectivity_by_survey,Year,sex_and_age"<<endl;
    for (int index=1;index<=Nsurveyindex;index++)
  {  
   out1<<surveyva(index)<<endl;
  }
  out1<<endl;
  out1<<endl;
  out1<<"**Length-structured_values**"<<endl<<endl;
  out1<<"Survey_length-specific_selectivity_by_survey,Year_and_length"<<endl;
  out1<<surveyvl<<endl<<endl<<endl;
  out1<<"**Biomass**"<<endl<<endl;
  out1<<"Vulnerable_Biomass_by_Method_and_Year"<<endl;
  out1<<B<<endl;
  out1<<"Spawning_Biomass_by_Year"<<endl;
  out1<<S<<endl;
  out1<<"Virgin_Spawning_Biomass"<<endl;
  out1<<S0<<endl<<endl<<endl;
  out1<<"**Exploitation_Rate**"<<endl<<endl;
  out1<<"Exploitation_Rate_by_Method_and_Year"<<endl;
  out1<<u<<endl<<endl<<endl;
  out1<<"**Numbers_at_age**"<<endl<<endl;
  out1<<"Numbers_at_age_by_Year,sex_and_age"<<endl;
  out1<<N<<endl<<endl<<endl;
  out1<<"**Length-at-age**"<<endl<<endl;
  out1<<"Length-at-age_by_sex"<<endl;
  out1<<LgivenA<<endl<<endl<<endl;
  out1<<"**Data_Fit**"<<endl<<endl;
  out1<<"Total_Catch_by_Method_and_Year"<<endl;
  out1<<C<<endl;
  out1<<"Number_of_CPUE_data"<<endl;
  out1<<NCPUE<<endl;
  out1<<"Observed_CPUE"<<endl;
  out1<<" index method year value cv"<<endl;
  out1<<CPUE<<endl;
  out1<<"Predicted_CPUE"<<endl;
  out1<<predCPUE<<endl;
  out1<<"Number_of_survey_data"<<endl;
  out1<<Nsurvey<<endl;
  out1<<"Observed_Survey"<<endl;
  out1<<" index year value cv"<<endl;
  out1<<survey<<endl;  
  out1<<"Predicted_Survey"<<endl;
  out1<<predsurvey<<endl;
  out1<<"Number_of_Commercial_C@A"<<endl;
  out1<<NCA<<endl;
  out1<<"Observed_C@A"<<endl;
  out1<<" method year samplesize sex1a1 sex1a2 sex1a3 "<<endl;
  out1<<CA<<endl;
  out1<<"Observed_C@A_pool"<<endl;
  out1<<CAobtemp<<endl;
  out1<<"Predicted_C@A"<<endl;
  out1<<CApredtemp<<endl;
  out1<<"Number_of_survey_C@A"<<endl;
  out1<<NsurveyCA<<endl;
  out1<<"Observed_C@A"<<endl;
  out1<<" survey year samplesize sex1a1 sex1a2 sex1a3 "<<endl;
  out1<<surveyCA<<endl;
  out1<<"Observed_C@A_pooled"<<endl;
  out1<<CAsurveyobtemp<<endl;
  out1<<"Predicted_C@A"<<endl;
  out1<<surveyCApredtemp<<endl;
  out1<<"Number_of_Commercial_C@L"<<endl;
  out1<<NCL<<endl;
  out1<<"Observed_C@L"<<endl;  
  out1<<" method year samplesize sex1l1 sex1l2 sex1l3 "<<endl;
  out1<<CL<<endl;
  out1<<"Predicted_C@L"<<endl;
  out1<<CLp<<endl;
  out1<<"Number_of_survey C@L"<<endl;
  out1<<NsurveyCL<<endl;
  out1<<"Observed_C@L"<<endl;  
  out1<<" survey year samplesize sex1l1 sex1l2 sex1l3 "<<endl;
  out1<<surveyCL<<endl;
  out1<<"Observed_C@L_pooled"<<endl;  
  out1<<CLsurveyobtemp<<endl;
  out1<<"Predicted_C@L"<<endl;
  out1<<surveyCLp<<endl;
  out1<<"Number_of_survey_C@L_with_no_sex"<<endl;
  out1<<NsurveynosexCL<<endl;
  out1<<"Observed_C@L"<<endl;
  out1<<" survey year samplesize l1 l2 l3 "<<endl;
  out1<<surveynosexCL<<endl;
  out1<<"Observed_C@L_pooled"<<endl;
  out1<<CLsurveynosexobtemp<<endl;
  out1<<"Predicted_C@L"<<endl;
  out1<<surveynosexCLp<<endl<<endl<<endl;
  out1<<"**Projection_Parameters**"<<endl<<endl;
  out1<<"StrategyType_1=constant_catch_2=harvest_rate"<<endl;
  out1<<StrategyType<<endl;
  out1<<"ProjectYear"<<endl;
  out1<<ProjectYear<<endl;
  out1<<"StartStrategy"<<endl;
  out1<<StartStrategy<<endl;
  out1<<"EndStrategy"<<endl;
  out1<<EndStrategy<<endl;
  out1<<"StepStrategy"<<endl;
  out1<<StepStrategy<<endl;
  out1<<"strategyuproportion by method"<<endl;
  out1<<strategyuproportion<<endl<<endl<<endl;
  out1<<"**Projection_Results**"<<endl<<endl;
  out1<<"Spawning Biomass by strategy"<<endl;
  out1<<Sproject<<endl;
  out1<<"**CPUE_Index_Trajectories**"<<endl<<endl;
  out1<<"CPUE_Index_Trajectories"<<endl;
  for(index=1;index<=NCPUEindex;index++)
  {
   out1<<elem_prod(B(CPUE(index,2)),qCPUE(index))<<endl; 
  }
  out1<<endl<<endl; 
  out1<<"**Survey_Index_Trajectories**"<<endl<<endl;
  out1<<"Survey_Index_Trajectories"<<endl;
  dvariable Btemp;
  for(index=1;index<=Nsurveyindex;index++)
    {
      for(int year=StartYear;year<=EndYear+1;year++)
        { 
          Btemp=0;
          if (survey_IndexType(index)==1)  //Index in weight 
            {
              if (survey_vType(index)==1)  // age-dependent sel
                {
                   for (int sex=1;sex<=Nsexes;sex++)
                     {
                       Btemp+=(elem_prod(N(year,sex),surveyva(index,sex))*w(year,sex))*surv05(sex);
                     }
                }
              else if (survey_vType(index)==2)  //size-dep sel
                {
                   for (int sex=1;sex<=Nsexes;sex++)
                     {
                       Btemp+=(elem_prod(N(year,sex),surveyva(index,sex))*survey_w(index,sex))*surv05(sex);
                     }
                }
           }
          else if (survey_IndexType(index)==2)  //Index in numbers
           {
             for (int sex=1;sex<=Nsexes;sex++)
               {
                 Btemp+=(N(year,sex)*surveyva(index,sex))*surv05(sex);
	       }
           }
         out1<<Btemp*qsurvey(index)<<" "; 
       }    // year
       out1<<endl;
   } //index
  out1<<endl<<endl; 

// Stock-Recruitment information
  out1<<"Data_for_the_Stock_-_Recruitment_graph"<<endl;
  out1<<"Equally_spaced_Spawning_Biomass"<<endl;
  dvariable S_graph;
  S_graph=0.0;
   for(int year=StartYear;year<=EndYear+1;year++)
    {
    out1<<S_graph<<" ";
    S_graph=S_graph+S0/(EndYear-StartYear);
    }
    out1<<endl;
  out1<<"Deterministic_recruitment"<<endl;
  S_graph=0.0;
  
//OjO : modifique esto  
   for(year=StartYear;year<=EndYear+1;year++)
    {
      if(active(log_RecDev))
         {   
           out1<<biasRannual*S_graph/(alpha+beta*S_graph)<<" ";
         }
        else
         {
           out1<<S_graph/(alpha+beta*S_graph)<<" ";
         }
      S_graph=S_graph+S0/(EndYear-StartYear);
    }
    out1<<endl;
  out1<<"Deterministic_Linear_recruitment"<<endl;
  S_graph=0.0;
   for(year=StartYear;year<=EndYear+1;year++)
    {
    	if(active(log_RecDev))
         {   
	    out1<<biasRannual*S_graph*(R0/S0)<<" ";
         }
        else
         {
 	    out1<<S_graph*(R0/S0)<<" ";
         }
      S_graph=S_graph+S0/(EndYear-StartYear);
    }
    out1<<endl;
  out1<<"Spawning_Biomass_by_Year"<<endl;
  for(year=StartYear;year<=EndYear;year++)
     {
       out1<<S(year)<<" ";
     }
     out1<<endl;

  out1<<"Recruitment_by_Year"<<endl;
  for(year=StartYear+1;year<=EndYear+1;year++)
     {
       if(Nsexes==1) out1<<N(year,1,1)<<" ";
       if(Nsexes==2) out1<<N(year,1,1)+N(year,2,1)<<" ";
     }
     out1<<endl;
  out1.close();
  
FUNCTION Likelihoods
   ofstream out2("likelihood.dat",ios::trunc);
   out2<<"**Likelihoods**"<<endl<<endl;
   out2<<"Total_likelihood "<<*objective_function_value::pobjfun<<endl;
   out2<<"Worst_gradient "<<objective_function_value::gmax<<endl;   
   out2<<"CPUE "<<CPUELL<<endl;
   out2<<"Survey_Index "<<SurveyLL<<endl;
   out2<<"C@A_Commercial "<<CALL<<endl;
   out2<<"C@L_Commercial "<<CLLL<<endl;
   out2<<"C@A_survey "<<surveyCALL<<endl;
   out2<<"C@L_no_sex_data_survey "<<surveynosexCLLL<<endl;
   out2<<"C@L_data_survey "<<surveyCLLL<<endl;
   out2<<"Priors "<<Priorlike<<endl;
   out2<<"Penalty_for_U "<<negpen<<endl;
   out2.close();  
  
//FINAL_SECTION
REPORT_SECTION
  printOldStuff(); 
  Output_File();
  Likelihoods();

//====================================================================================================================================================

RUNTIME_SECTION
  //convergence_criteria .000001 
  maximum_function_evaluations 400,1000,2000,2000

//====================================================================================================================================================




TOP_OF_MAIN_SECTION
  arrmblsize = 5000000;
  gradient_structure::set_GRADSTACK_BUFFER_SIZE(560000);
  gradient_structure::set_CMPDIF_BUFFER_SIZE(15000000);
  gradient_structure::set_MAX_NVAR_OFFSET(500);
  gradient_structure::set_NUM_DEPENDENT_VARIABLES(500);

//====================================================================================================================================================
GLOBALS_SECTION
  #include <admodel.h>
  long int rec_seed;
  long int assessment_seed;

  ofstream proj1("mcmclike.out"); // proj1 <- Print all the likelihoods to the file "mcmclike.out"
  ofstream proj2("params.pst");   // proj2 <- Print out all parameters, but not residuals
  ofstream proj3("resids.pst");   // proj3 <- Print out all residuals
  ofstream proj4("spawBiom.pst"); // proj4 <- spawning Biomass
  ofstream proj5("expRate.pst");  // proj5 <- exploitation rate
  ofstream proj6("vulnBiom.pst"); // proj6 <- vulnerable Biomass 
  ofstream proj7("Recruits.pst"); // proj7 <- recruits
  ofstream proj8("Ro_So_VVB.pst");// proj8 <- Virgin Recruitment / Virgin Spawning Biomass / Virgin Vulnerable Biomass


  ofstream proj9("ProjBiom.out");
  ofstream proj10("ProjSpBm.out");
  ofstream proj11("ProCatch.out");
  ofstream proj12("RecRes.out");
  ofstream proj13("Strategy.out");

		  

//  ofstream report1("project.out");


//**************** LIKELIHOOD GENERAL FUNCTION *************************
  dvariable nlikelihood (dvector& observed, dvar_vector expected, dvector& cv, int LikeType)
    {
      RETURN_ARRAYS_INCREMENT();
      dvariable logLike;
      switch (LikeType)
        {
          case 0: //not used
            break;
          case 1: //normal
            logLike=sum(elem_div(square(observed-expected) , 2 * square(elem_prod(cv,observed)))); 
	    break;
          case 2: //lognormal
            logLike=sum(elem_div(square(log(observed)-log(expected)) , 2*square(cv))); 
            break;
          default: 
            cout<<"No such likelihood: "<<LikeType<<" /n";
          break;
      }
   
       RETURN_ARRAYS_DECREMENT();
       return(logLike);
     }


//**************** LIKELIHOOD GENERAL FUNCTION *************************
  dvariable likelihood (double observed, dvariable expected, double cv, int LikeType)
  {
   RETURN_ARRAYS_INCREMENT();
   dvariable logLike;
   double pi=3.14159265358979;
   switch (LikeType)
   {
   case -1: //lognormal
      logLike=square(log(observed)-log(expected))/(2*square(cv)); // +  log(cv)+  0.5*log(2*pi);
      break;
   case 0: //not used
      break;
   case 1: //normal
      logLike=square((observed)-(expected))/(2*square(cv*observed)); // +  log(cv*observed)+  0.5*log(2*pi);
      break;
   case 2: //lognormal
      logLike=square(log(observed)-log(expected))/(2*square(cv)); // +  log(cv)+  0.5*log(2*pi);
      break;
   case 3: //robust normal need constant term
      logLike=-log( exp( -1*square((observed)-(expected))/(2*square(cv*observed)))+0.01  );
      break;
   case 4: //robust lognormal need constant term
      logLike=-log(exp(-1*square(log(observed)-log(expected))/(2*square(cv)))+0.01);
      break;
   default: 
      cout<<"No such likelihood: "<<LikeType<<" /n";
      break;
   }
  RETURN_ARRAYS_DECREMENT();
  return(logLike);
  }

//********************** PRIOR GENERAL FUNCTION ********************************
  dvariable prior (dvariable parameter_value, dvector& prior_dat)
  {
  RETURN_ARRAYS_INCREMENT();
   dvariable prior_value;
   double pi=3.14159265358979;
   int switch_temp=prior_dat(4);
   switch (switch_temp)
   {
   case 0: //uniform
      //do nothing
      prior_value=0;
      break;
   case 1: //normal
      //check if realy log normal
      if (prior_dat(5)==0) prior_value=square((prior_dat(5))-(parameter_value))/(2*square(prior_dat(6)));// +  log(prior_dat(6))+  0.5*log(2*pi);
      else prior_value=square((prior_dat(5))-(parameter_value))/(2*square(prior_dat(6)*prior_dat(5)));// +  log(prior_dat(6)*prior_dat(5))+  0.5*log(2*pi);
      break;
   case 2: //lognormal
      prior_value=square(log(prior_dat(5))-log(parameter_value))/(2*square(prior_dat(6)));// +  log(prior_dat(6))+  0.5*log(2*pi);
      break;
   default:
      cout<<"No such prior: "<<prior_dat(4)<<" Will be uniform";
      break;
   }
  RETURN_ARRAYS_DECREMENT();
  return(prior_value);
  }

//************************ PRINT PRIORS ******************************
  void prior_print(char *parameter_name, dvariable parameter_value, dvector& prior_dat)
  {
  char string[10];
  strcpy( string, parameter_name );
  //cout<<parameter_name<<":   ";
  int temp=strlen(string);
  for (int i=1; i<=10-temp;i++) strcat(string," ");
  cout<<string<<":   ";
   if (prior_dat(1)<0) cout<<"Fixed    "<<parameter_value;
   else               
   {
      cout<<"Phase="<<prior_dat(1);
      int switch_temp=prior_dat(4);
      switch (switch_temp)
      {
      case 0:                    
         cout<<"  Uniform    "<<"LB="<<prior_dat(2)<<"  UB="<<prior_dat(3)<<"  Pin="<<prior_dat(7);
         break;
      case 1:
         cout<<"  Normal     "<<"Mean="<<prior_dat(5)<<"  CV="<<prior_dat(6)<<"  LB="<<prior_dat(2)<<"  UB="<<prior_dat(3)<<"  Pin="<<prior_dat(7);
         break;
      case 2:  
         cout<<"  Lognormal  "<<"Mean="<<prior_dat(5)<<"  CV="<<prior_dat(6)<<"  LB="<<prior_dat(2)<<"  UB="<<prior_dat(3)<<"  Pin="<<prior_dat(7);
         break;
      default:
         cout<<"  No such prior: "<<prior_dat(4)<<" for "<<parameter_name<<". Will be uniform  "<<"LB="<<prior_dat(2)<<"  UB="<<prior_dat(3)<<"  Pin="<<prior_dat(7);
         break;
      }
   } 
   cout<<" \n";
  }
 
 //************************ PRINT Likelihoods ******************************
  void like_print(char *parameter_name,int index, int LikeType, double cv)
  {
  char string[13];
  strcpy( string, parameter_name );
  //cout<<parameter_name<<":   ";
  int temp=strlen(string);
  for (int i=1; i<=13-temp;i++) strcat(string," ");
  cout<<string<<" "<<index<<": ";
   switch (LikeType)
   {
   case 0:                    
      cout<<"Not Used";
      break;
   case 1:
      cout<<"Normal     cv = "<<cv;
      break;
   case 2:  
      cout<<"Lognormal  cv = "<<cv;
      break;
   case 3:  
      cout<<"Robust Normal??  cv = "<<cv;
      break;
   case 4:  
      cout<<"Robust Lognormal??  cv = "<<cv;
      break;
   case 11:  
      cout<<"Multinomial not implemented  cv = ";
      break;
   case 12:  
      cout<<"Robust Lognormal for propotions";
      break;
   default:
      cout<<"No such Likelihood "<<LikeType;
      break;
   }
   //cout<<" \n";
  }

 
//=================================================================================  
// AGE/LENGTH STRUCTURED LIKELIHOODS
//=================================================================================  

//=================================================================================  
// 10 : Multinomial likelihood 
//=================================================================================  
  dvariable multinomial(dmatrix& obs ,dvar_matrix& pred, double& a , int& nobserv , dvector& b)  
    {
      RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
         dmatrix bv(1,nobserv,1,a);
         double sum_offset;
	 
   	 for (int i=1;i<=nobserv;i++)
           {
             bv(i) = b(i);
           }	 
	 sum_offset = sum(-1.0*elem_prod(bv,elem_prod((obs + 0.001) , log(obs + 0.001))));
         dvar_matrix v =elem_prod(bv,elem_prod((obs + 0.001) , log(pred + 0.001)));
         dvariable log_likelihood = -1.0 * sum(v) - sum_offset;;
      RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
                              // caused by RETURN_ARRAYS_INCREMENT();
      return(log_likelihood);
    }
//=================================================================================  
// 12 : David Fournier's robust normal likelihood for proportions 
//=================================================================================  
  dvariable nrobust_p (dmatrix& obs ,dvar_matrix& pred, double& a , int& nobserv , dvector& b) //,vector& b 
    {
      RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
         dmatrix bv(1,nobserv,1,a);
	 for (int i=1;i<=nobserv;i++)
           {
             bv(i) = b(i);
           }	 
	 dvar_matrix v = (elem_prod(obs ,1. - obs )+ .1/a);
         dvar_matrix l = exp(-0.5* elem_prod(bv , elem_div(square(obs - pred), v )));
         dvariable log_likelihood = -1.0*sum(log(l + .01));
         log_likelihood += 0.5 * sum(log(v));
      RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
                              // caused by RETURN_ARRAYS_INCREMENT();

      return(log_likelihood);
    }

//=================================================================================  
// THIS SECTION IS FOR CONSTRAINING THE EXPLOITATION RATE
//=================================================================================  
  dvar_vector bound_at_one(_CONST dvar_vector& t1)
    {
      RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
       int t2=t1.indexmax();
       int size = t1.size();
       double thresh = 0.99;
       double k = 50;
       dvar_vector tmp(1,size);
       tmp= t1;
       for (int i=1;i<=t2;i++)
          {
           if (tmp(i) > thresh)
             {
               tmp(i)=thresh+(1.-thresh)*(1.-exp(-k*(tmp(i)-thresh)));
             }
         }
      RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
      return tmp;
    }

//=================================================================================  
  dvariable bound_at_one(_CONST dvariable& t1)
    {
      RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
       double thresh = 0.99;
       double k = 50;
       dvariable tmp;
       tmp = t1;
       if (tmp > thresh)
         {
           tmp=thresh+(1.-thresh)*(1.-exp(-k*(tmp-thresh)));
         }
      RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
      return tmp;
    }

//=================================================================================  
  double bound_at_one(_CONST double& t1)
     {
       double thresh = 0.99;
       double k = 50;
       double tmp;
       tmp = t1;
       if (tmp > thresh)
         {
       tmp=thresh+(1.-thresh)*(1.-exp(-k*(tmp-thresh)));
         }

       return tmp;
     }

//=================================================================================  
//THIS ADDS A PENALTY
//=================================================================================  
//####################################################################### 
  dvariable posfun (_CONST dvariable& x,_CONST double eps, dvariable& pen)
    {
      if (x>=eps)
        { 
           return x;
        }
      else
        {
           dvariable y=1.0 -x/eps;
       	   pen+=.01*square(x-eps);
       	   return eps*(1./(1.+y+y*y));
       }
    }

//####################################################################### 
  dvariable posfan(_CONST dvariable& x, dvariable& eps, dvariable& penalty,_CONST double bound2 )
    {
      if (x>=0.03)
        {
          return x;
        }
      else
        {
          dvariable y = 1.0 - x/0.03;
          penalty += 0.05*square((bound2-eps)-0.03);
          return  0.03*(1./(1.+y+y*y));
        }
    }  
//===================================================================================================================================  
//variance of length at age
//===================================================================================================================================  
  dmatrix Calc_LgivenA(_CONST int& Nages,_CONST int& Nlens, dvector real_length, dvector& Lmean, dvector& Lcv, _CONST int& inkL)
    {
      RETURN_ARRAYS_INCREMENT(); 
       dmatrix LgivenA(1,Nages,1,Nlens);
       double lensum; 
       double log_inkL;
       dvector obs(1,Nlens);
         obs=log(real_length);
       dvector pred(1,Nages);
         pred=log(Lmean)-square(Lcv)/2;
       //Lenght given Age
       for (int age=1;age<=Nages;age++)
         {
           char mark;
           lensum=0;
           for (int length=1;length<=Nlens;length++)
              {
                log_inkL=log(real_length(length)+0.5*inkL)-log(real_length(length)-0.5*inkL);
                LgivenA(age,length)=exp(-0.5*square(  (obs(length)-pred(age))/Lcv(age) ))*log_inkL;     
              }
           lensum=sum(LgivenA(age));
           LgivenA(age)=LgivenA(age)/lensum;
        }
       RETURN_ARRAYS_DECREMENT(); 
       return(LgivenA);
     }

//===========================================================================================================================================  
// Selectivity at age
//===========================================================================================================================================  
  dvar_vector selage(_CONST dvariable& full ,_CONST dvariable& varL ,_CONST dvariable& varR , _CONST int& Nages)
    {
      RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
        dvar_vector sel(1,Nages);   
        for (int age=1;age<=Nages;age++)
           {
             if (full>age)
               {
                 sel(age)=exp(-1.0*square(age-full)/varL);             
               }
             else
               {
                 sel(age)=exp(-1.0*square(age-full)/varR);             
               }
           }
      RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
      return(sel);
    }
  
//===========================================================================================================================================  
// Selectivity at length 
//===========================================================================================================================================  
 
  dvar_vector sellen(_CONST dvariable& full ,_CONST dvariable& varL ,_CONST dvariable& varR , _CONST dvector& real_length , _CONST int& Nlens)
    {
       RETURN_ARRAYS_INCREMENT(); //Need this statement because the function
       dvar_vector sel(1,Nlens);   
       for (int len=1;len<=Nlens;len++)
          {
            if (full>real_length(len))
              {
                sel(len)=exp(-1.0*square(real_length(len)-full)/varL);                
              }
            else
              {
                sel(len)=exp(-1.0*square(real_length(len)-full)/varR);                
              }
          }
       RETURN_ARRAYS_DECREMENT(); // Need this to decrement the stack increment
       return(sel);
    }

//===========================================================================================================================================  
// END of PROGRAM 
//===========================================================================================================================================  
